package edu.wpi.aipaint.ai;

import java.util.Iterator;
import java.util.List;
import java.util.TreeMap;

import edu.wpi.aipaint.attributes.Attribute;
import edu.wpi.aipaint.main.WorldState;
import edu.wpi.aipaint.pipeline.Action;
import edu.wpi.aipaint.values.Value;

public class BTNominalDecisionNode implements IBTDecisionNode {
	protected Attribute attribute;
	protected IBTDecisionNode parent;
	protected List<Value> decisionSlots;
	protected TreeMap<Value, IBTNode> children;
	protected Value minValue, maxValue;

	public BTNominalDecisionNode(
			IBTDecisionNode parent, Attribute attribute,
			List<Value> decisionSlots) {
		this.parent = parent;
		this.attribute = attribute;
		this.children = new TreeMap<Value, IBTNode>();
		this.decisionSlots = decisionSlots;
	}

	public IBTDecisionNode attachNextChild(	IBTNode p_child,
											WorldState p_ws)
			throws DiscourseException {
		if (!decisionSlots.isEmpty()) {
			Value val = decisionSlots.remove(0);

			this.children.put(val, p_child);

			return this;
		} else {
			IBTNode child = this.children.get(this.attribute.evaluate(p_ws));
			if (child instanceof IBTLeafNode) {
				throw new DiscourseException("No free spots found!",
						this,
						DiscourseException.TYPE.NO_AVAILABLE_MERGE);
			} else {
				return ((IBTDecisionNode) child).attachNextChild(p_child, p_ws);
			}
		}
	}

	@Override
	public Action evaluate(WorldState p_ws) {
		IBTNode child = this.getChild(p_ws);
		if (child instanceof IBTDecisionNode) {
			return ((IBTDecisionNode) child).evaluate(p_ws);
		} else if (child instanceof IBTLeafNode) {
			return ((IBTLeafNode) child).getAction();
		} else {
			System.err.println("There is no valid move");
			return null;
		}
	}

	@Override
	public Attribute getAttribute() {
		// TODO Auto-generated method stub
		return null;
	}

	public IBTNode getChild(Value p_attributeValue) {
		return this.children.get(p_attributeValue);
	}

	@Override
	public IBTNode getChild(WorldState p_ws) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public int getChildCount() {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public Iterator<? extends IBTNode> getChildrenIterator() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public List<? extends Value> getDecisionSlots() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Value getLowerBound(IBTNode p_child) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public IBTDecisionNode getParent() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Value getUpperBound(IBTNode p_child) {
		// TODO Auto-generated method stub
		return null;
	}

	public Value getValue(IBTNode p_child) {
		for (Value v : this.children.keySet()) {
			if (this.getChild(v) == p_child) {
				return v;
			}
		}
		return null;
	}

	@Override
	public IBTDecisionNode insertChild(	Value p_value,
										IBTNode p_child) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void mergeChildren(IBTDecisionNode p_other)
			throws DiscourseException {
		// TODO Auto-generated method stub

	}

	@Override
	public IBTNode removeChild(IBTNode p_child) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public IBTNode removeChild(Value p_value) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void setParent(IBTDecisionNode parent) {
		// TODO Auto-generated method stub

	}

	public String toXML() {
		throw new UnsupportedOperationException();
	}

	public String treePrint(int p_depth) {
		String ret = "";
		for (Value f : this.decisionSlots) {
			for (int i = 0; i < p_depth; i++) {
				ret += "| ";
			}
			ret += "if " + this.attribute.toString() + " == ";
			ret += f;
			ret += " do\n";
			ret += this.children.get(f).treePrint(p_depth + 1);
		}
		return ret;
	}
}