package edu.wpi.aipaint.attributes;

import edu.wpi.aipaint.main.WorldState;
import edu.wpi.aipaint.util.Vector2;
import edu.wpi.aipaint.values.BooleanWrapper;
import edu.wpi.aipaint.values.Value;
import edu.wpi.aipaint.values.Vector2Wrapper;

/**
 * When evaluated, computes whether the value of a Vector2 Attribute lies 
 * within a particular rectangular area.
 * 
 * The Value returned is a BooleanWrapper.
 */
public class AreaAttribute implements Attribute {
	private Attribute attributeToTest;
	private Vector2 nearCorner;
	private Vector2 farCorner;
	
	/**
	 * @param attributeToTest
	 * @param nearCorner One corner of the target rectangular area.
	 * @param farCorner The opposite corner of the target rectangular area.
	 */
	public AreaAttribute(Attribute attributeToTest, Vector2 nearCorner, Vector2 farCorner)
	{
		this.attributeToTest = attributeToTest;
		this.nearCorner = nearCorner;
		this.farCorner = farCorner;
	}

	@Override
	public Value evaluate(WorldState worldState) {
		Vector2 value = ((Vector2Wrapper)attributeToTest.evaluate(worldState)).getValue();
		boolean result = value.getX() >= nearCorner.getX() &&
							value.getY() >= nearCorner.getY() &&
							value.getX() <= farCorner.getX() &&
							value.getY() <= farCorner.getY();
		return new BooleanWrapper(result);
	}
}
