package edu.wpi.aipaint.attributes;

import edu.wpi.aipaint.main.Group;
import edu.wpi.aipaint.main.Poseable;
import edu.wpi.aipaint.main.WorldState;
import edu.wpi.aipaint.util.Vector2;
import edu.wpi.aipaint.values.Value;
import edu.wpi.aipaint.values.Vector2Wrapper;
import edu.wpi.aipaint.worldspacepoint.WorldSpacePoint;

/**
 * Examines the Vector2-valued WorldFeatures of the Poseables belonging to a 
 * particular Group and returns the Value of the WorldFeature closest to a 
 * particular WorldSpacePoint.
 */
public class ClosestAttribute implements Attribute {
	WorldSpacePoint point;
	Group group;
	
	public ClosestAttribute(WorldSpacePoint point, Group group)
	{
		this.point = point;
		this.group = group;
	}
	
	@Override
	public String toString()
	{
		return "ClosestAttribute [" + group.toString() + " closest to " + point.toString() + "]";
	}

	@Override
	public Value evaluate(WorldState worldState) {
		Iterable<Poseable> groupMembers = group.getMembers();
		
		float minDistance = -1;
		Poseable closestPoseable = null;
		
		for (Poseable groupMember : groupMembers)
		{
			Vector2 position = ((Vector2Wrapper)worldState.getValue(
					groupMember.getFeature())).getValue();
			float distance = Math.abs(point.getX() - position.getX()) + 
				Math.abs(point.getY() - position.getY());
			
			if (distance < minDistance || minDistance < 0)
			{
				minDistance = distance;
				closestPoseable = groupMember;
			}
		}
		
		return worldState.getValue(closestPoseable.getFeature());
	}
}
