package edu.wpi.aipaint.attributes;

import edu.wpi.aipaint.main.WorldState;
import edu.wpi.aipaint.util.Vector2;
import edu.wpi.aipaint.values.FloatWrapper;
import edu.wpi.aipaint.values.Value;
import edu.wpi.aipaint.values.Vector2Wrapper;

/**
 * When evaluated, computes the distance between two Attributes of type
 * Vector2.
 */
public class DistanceAttribute implements Attribute {
	private Attribute position1;
	private Attribute position2;
	
	// This variable is not used by this class.
	// It is stored here as a way to enable rendered
	// DistanceConditionalStatements to easily find out what their threshold 
	// value was before they were translated.
	// (In other words, it's a hack for demoing purposes.)
	private float threshold;
	
	public DistanceAttribute(Attribute position1, Attribute position2)
	{
		this.position1 = position1;
		this.position2 = position2;
	}

	@Override
	public Value evaluate(WorldState worldState) {
		Vector2 vector1 = ((Vector2Wrapper)position1.evaluate(worldState)).getValue();
		Vector2 vector2 = ((Vector2Wrapper)position2.evaluate(worldState)).getValue();
		float distance = (float) Math.sqrt(Math.pow(vector2.getX() - vector1.getX(), 2) + 
				Math.pow(vector2.getY() - vector1.getY(), 2));
		return new FloatWrapper(distance);
	}
	
	public Attribute getPosition1()
	{
		return position1;
	}
	
	public Attribute getPosition2()
	{
		return position2;
	}
	
	public String toString() {
		return "DistanceAttribute ["+ this.position1 + " to " + this.position2 + "]";
	}
	
	// BEGIN HAXX
	public void setThreshold(float threshold) {
		this.threshold = threshold;
	}
	public float getThreshold() {
		return threshold;
	}
	// END HAXX
}
