package edu.wpi.aipaint.directo;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import edu.wpi.aipaint.main.MainWindow;
import edu.wpi.aipaint.pipeline.Direction;
import edu.wpi.aipaint.pipeline.PipelineModule;
import edu.wpi.aipaint.pipeline.Statement;
import edu.wpi.aipaint.pipeline.StatementProcessor;
import edu.wpi.aipaint.util.UserObjectTreeNode;

/**
 * Translates a collection of Statements into a collection of Directions that
 * describe modifications to an AI data representation. 
 */
public class BTDirectoModule implements StatementProcessor {
	
	private PipelineModule parent;
	
	@Override
	public void processStatements(UserObjectTreeNode<Statement> statementTree)
	{
		List<Direction> directions = new LinkedList<Direction>();
		
		// Turn Statements into Directions.
		// We execute a preorder traversal of the tree.
		directions.addAll(translateStatementTree(statementTree));
		
		// At some point, call StatementProcessingDone on the 
		// parent pipeline module.
		// (You might want to perform discourse outside of this method first.)
		parent.statementProcessingDone(directions);
	}
	
	/**
	 * Performs a preorder traversal of the given tree of Statements,
	 * translating each Statement into a List of Directions and returning
	 * the collected Lists.
	 * @param statementTree
	 */
	@SuppressWarnings("unchecked")
	private List<Direction> translateStatementTree(UserObjectTreeNode<Statement> statementTree)
	{
		List<Direction> translation = new LinkedList<Direction>();
		
		// Translate the root node of the given tree.
		translation.addAll(statementTree.getUserObject().translate());
		
		// Translate the children from left to right.
		Iterator<UserObjectTreeNode<Statement>> childIterator = 
			(Iterator<UserObjectTreeNode<Statement>>) statementTree.getChildren();
		while (childIterator.hasNext())
			translation.addAll(translateStatementTree(childIterator.next()));
		
		return translation;
	}

	@Override
	public void activate(MainWindow window) { }

	@Override
	public void deactivate() { }

	@Override
	public String getName() {
		return "Behavior Tree Directo Module";
	}

	@Override
	public void tick() { }

	@Override
	public void setPipelineParent(PipelineModule parent) {
		this.parent = parent;
	}
}
