package edu.wpi.aipaint.main;

import java.util.HashMap;

/**
 * Singleton class that constructs and holds Group instances and registers
 * Poseables as Group members. 
 */
public class GroupManager {
	private static GroupManager instance = null;
	
	private HashMap<String, Group> groupsByName;
	
	private GroupManager()
	{
		this.groupsByName = new HashMap<String, Group>();
	}
	
	public static GroupManager GetInstance()
	{
		if (instance == null)
			instance = new GroupManager();
		
		return instance;
	}
	
	/**
	 * Returns the Group with the given name or null if there is no such Group.
	 * @param groupName
	 * @return
	 */
	public Group getGroupByName(String groupName)
	{
		return groupsByName.get(groupName);
	}
	
	/**
	 * Creates a Group with the given name.
	 * Throws an Exception if a Group with this name already exists.
	 * @param groupName
	 * @return
	 * @throws Exception
	 */
	public Group createGroup(String groupName)
	{
		if (!groupsByName.containsKey(groupName))
		{
			Group newGroup = new Group(groupName);
			groupsByName.put(groupName, newGroup);
			return newGroup;
		}
		
		throw new RuntimeException("Group '" + groupName + "' already exists!");
	}
	
	/**
	 * Registers the given Poseable as a member of the Group with the given
	 * name.  Returns the Group with which the Poseable registered.
	 * Throws an Exception if no Group with the given name currently exists.
	 * @param poseable
	 * @param groupName
	 * @return
	 */
	public Group registerPoseable(Poseable poseable, String groupName)
	{
		if (groupsByName.containsKey(groupName))
		{
			Group requestedGroup = groupsByName.get(groupName);
			requestedGroup.addMember(poseable);
			return requestedGroup;
		}
		
		throw new RuntimeException("Group '" + groupName + "' does not exist!");
	}
}
