package edu.wpi.aipaint.main;

import java.awt.Component;
import java.awt.Container;
import java.awt.Point;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.JPanel;
import javax.swing.SwingUtilities;

public class InputListenerProxy implements MouseListener, MouseMotionListener, KeyListener {
	MainWindow window;
	JPanel sideBar;
	Container contentPane;
	Component glassPane;
	MouseListener subMouseListener;
	MouseListener gameMouseListener;
	MouseMotionListener subMouseMotionListener;
	MouseMotionListener gameMouseMotionListener;
	KeyListener subKeyListener;
	KeyListener gameKeyListener;
	
	public InputListenerProxy(MainWindow window, JPanel sideBar, 
			Container contentPane, Component glassPane) {
		this.window = window;
		this.sideBar = sideBar;
		this.contentPane = contentPane;
		this.glassPane = glassPane;
		this.subMouseListener = new NullMouseListener();
		this.gameMouseListener = new NullMouseListener();
		this.subMouseMotionListener = new NullMouseMotionListener();
		this.gameMouseMotionListener = new NullMouseMotionListener();
		this.subKeyListener = new NullKeyListener();
		this.gameKeyListener = new NullKeyListener();
	}
	
	public void setGlassPane(Component glassPane)
	{
		this.glassPane = glassPane;
	}
	
	public void setMouseListener(MouseListener subMouseListener)
	{
		this.subMouseListener = subMouseListener;
	}
	
	public void setGameMouseListener(MouseListener gameMouseListener)
	{
		this.gameMouseListener = gameMouseListener;
	}
	
	public void setMouseMotionListener(MouseMotionListener subMouseMotionListener)
	{
		this.subMouseMotionListener = subMouseMotionListener;
	}
	
	public void setGameMouseMotionListener(MouseMotionListener gameMouseMotionListener)
	{
		this.gameMouseMotionListener = gameMouseMotionListener;
	}
	
	public void setKeyListener(KeyListener subKeyListener)
	{
		this.subKeyListener = subKeyListener;
	}
	
	public void setGameKeyListener(KeyListener gameKeyListener)
	{
		this.gameKeyListener = gameKeyListener;
	}
	
	@Override
	public void mouseClicked(MouseEvent arg0) {
		if (window.isAIPaintPaused() && !redispatchMouseEvent(arg0))
			subMouseListener.mouseClicked(arg0);
		else
			gameMouseListener.mouseClicked(arg0);
	}

	@Override
	public void mouseEntered(MouseEvent arg0) {
		if(window.isAIPaintPaused() && !redispatchMouseEvent(arg0))
			subMouseListener.mouseEntered(arg0);
		else
			gameMouseListener.mouseEntered(arg0);
	}

	@Override
	public void mouseExited(MouseEvent arg0) {
		if(window.isAIPaintPaused() && !redispatchMouseEvent(arg0))
			subMouseListener.mouseExited(arg0);
		else
			gameMouseListener.mouseExited(arg0);
	}

	@Override
	public void mousePressed(MouseEvent arg0) {
		if(window.isAIPaintPaused() && !redispatchMouseEvent(arg0))
			subMouseListener.mousePressed(arg0);
		else
			gameMouseListener.mousePressed(arg0);
	}

	@Override
	public void mouseReleased(MouseEvent arg0) {
		if(window.isAIPaintPaused() && !redispatchMouseEvent(arg0))
			subMouseListener.mouseReleased(arg0);
		else
			gameMouseListener.mouseReleased(arg0);
	}

	@Override
	public void mouseDragged(MouseEvent arg0) {
		if(window.isAIPaintPaused() && !redispatchMouseEvent(arg0))
			subMouseMotionListener.mouseDragged(arg0);
		else
			gameMouseMotionListener.mouseDragged(arg0);
	}

	@Override
	public void mouseMoved(MouseEvent arg0) {
		if(window.isAIPaintPaused() && !redispatchMouseEvent(arg0))
			subMouseMotionListener.mouseMoved(arg0);
		else
			gameMouseMotionListener.mouseMoved(arg0);
	}
	
	//Adapted from http://download.oracle.com/javase/tutorial/uiswing/components/rootpane.html
	private boolean redispatchMouseEvent(MouseEvent e) {
		if(subMouseMotionListener == null || subMouseListener == null)
			return true;
		Point glassPanePoint = e.getPoint();
		Container container = contentPane;
		Point containerPoint = SwingUtilities.convertPoint(
              glassPane,
              glassPanePoint,
              contentPane);

		if (containerPoint.y < 0) { //we're not in the content pane
			//	Could have special code to handle mouse events over
			//	the menu bar or non-system window decorations, such as
			//the ones provided by the Java look and feel.
		} else {
			//		The mouse event is probably over the content pane.
			//	Find out exactly which component it's over.
			Component component =
				SwingUtilities.getDeepestComponentAt(
						container,
						containerPoint.x,
						containerPoint.y);

			if ((component != null) && belongsToItem(component, sideBar)) {
				//	Forward events over the check box.
				Point componentPoint = SwingUtilities.convertPoint(
						glassPane,
						glassPanePoint,
						component);
				component.dispatchEvent(new MouseEvent(component,
                           	e.getID(),
                           	e.getWhen(),
                           	e.getModifiers(),
                           	componentPoint.x,
                           	componentPoint.y,
                           	e.getClickCount(),
                           	e.isPopupTrigger()));
				return true;
			}
		}
		
		return false;
	}
	
	private boolean belongsToItem(Component child, Component suspectParent) {
		if(child.equals(suspectParent))
			return true;
		Component parent = child.getParent();
		while(parent != null && !parent.equals(suspectParent)) {
			parent = parent.getParent();
		}
		return parent != null;
	}

	@Override
	public void keyPressed(KeyEvent arg0) {
		if (KeyEvent.getKeyText(arg0.getKeyCode()).equals("K"))
			window.toggleAIPaintPause();
		
		if (window.isAIPaintPaused())
			subKeyListener.keyPressed(arg0);
		else
			gameKeyListener.keyPressed(arg0);
	}

	@Override
	public void keyReleased(KeyEvent arg0) {
		if (window.isAIPaintPaused())
			subKeyListener.keyReleased(arg0);
		else
			gameKeyListener.keyReleased(arg0);
	}

	@Override
	public void keyTyped(KeyEvent arg0) {
		if (window.isAIPaintPaused())
			subKeyListener.keyTyped(arg0);
		else
			gameKeyListener.keyTyped(arg0);
	}
}
