package edu.wpi.aipaint.main;

/**
 * Represents a feature of the game world, independent of the value
 * of the feature in any particular world state.
 */
public class WorldFeature {
	private String name;
	private boolean nominal;
	
	/**
	 * Constructs a continuous-valued WorldFeature with the given name.
	 * @param name
	 */
	public WorldFeature(String name) {
		super();
		this.name = name;
		this.nominal = false;
	}

	/**
	 * @return Name of WorldFeature
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return Is the feature nominal?
	 */
	public boolean isNominal() {
		return nominal;
	}

	@Override
	public String toString() {
		return "WorldFeature [name=" + name + ", nominal=" + nominal + "]";
	}
	
	/**
	 * Two WorldFeatures are equal if they have the same name.
	 */
	@Override
	public boolean equals(Object obj)
	{
		WorldFeature other = (WorldFeature) obj;
		return other.getName().equals(name);
	}
	
	@Override
	public int hashCode()
	{
		return name.hashCode();
	}
}
