package edu.wpi.aipaint.main;

import java.util.HashMap;

import edu.wpi.aipaint.values.Value;

/**
 * Represents a mapping of WorldFeatures to their corresponding values
 * at a particular instant.  This mapping defines a state of the game 
 * world on which AI agents may make decisions.
 */
public class WorldState {
	private HashMap<WorldFeature, Object> featureSet;
	
	public WorldState()
	{
		featureSet = new HashMap<WorldFeature, Object>();
	}

	/**
	 * Titles for displaying the state
	 * 
	 * @return Array of titles matching the WorldState
	 */
	public WorldFeature[] getFeatureSet()
	{
		WorldFeature[] ret = new WorldFeature[featureSet.size()];
		featureSet.keySet().toArray(ret);
		return ret;
	}

	/**
	 * Retrieves a specific feature
	 * 
	 * @param wf
	 *            WorldFeature to search for
	 * @return null if not found, value otherwise
	 */
	@SuppressWarnings("unchecked")
	public <AttributeType extends Value> AttributeType getValue(WorldFeature wf)
	{
		return (AttributeType) featureSet.get(wf);
	}

	/**
	 * Adds a new feature to the feature set.
	 * 
	 * @param wf
	 *            World Feature to add
	 * @param val
	 *            Value to assign it
	 */
	public <AttributeType extends Value> void addFeature(WorldFeature wf, AttributeType val)
	{
		featureSet.put(wf, val);
	}
	
	/**
	 * @param name WorldFeature to fetch
	 * @return the WorldFeature
	 */
	public WorldFeature getFeatureByName(String name)
	{
		for(WorldFeature wf : featureSet.keySet()){
			if(wf.getName().equals(name)){
				return wf;
			}
		}
		return null;
	}
	
	/**
	 * @return The full feature set, with all set values.
	 */
	public String displayFullFeatureSet()
	{
		String ret = this.toString() + "\n";
		for (WorldFeature wf : featureSet.keySet()) {
			ret += "\t" + wf + "\t-\t" + featureSet.get(wf) + "\n";
		}

		return ret;
	}
	
	/**
	 * Generate the next world state, with all values null
	 * @return Clears all values
	 */
	public WorldState copy()
	{
		WorldState cp = new WorldState();
		for(WorldFeature wf : featureSet.keySet()){
			cp.addFeature(wf, null);
		}
		return cp;
	}
}
