package edu.wpi.aipaint.shapedetector;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * 
 * A set of strokes representing a shape.
 * $N implementation - http://depts.washington.edu/aimgroup/proj/dollar/ndollar.html
 * Slightly modified for control point usage
 * @author amejia
 *
 */
public class MultiStroke {
	String Name;
	int NumStrokes;
	Template[] Templates;
	
	public MultiStroke(String name, boolean useLimitedRotationInvariance, Point[][] strokes, Point[] controlPoints) {
		this.Name = name;
		this.NumStrokes = strokes.length;
		
		int[] order = new int[strokes.length];
		for(int i = 0; i < strokes.length; i++)
			order[i] = i;
		
		ArrayList<OrderPermutation> orders = new ArrayList<OrderPermutation>();
		
		// Come up with every possible draw order of this shape (backwards and forwards for each stroke) and
		// save each permutation as a template
		
		
		HeapPermute(strokes.length, order, orders);
		
		ArrayList<ArrayList<Point>> unistrokes = MakeUnistrokes(strokes, orders);
		ArrayList<Point> cPoints = new ArrayList<Point>(Arrays.asList(controlPoints));
		this.Templates = new Template[unistrokes.size()];
		for(int j = 0; j < unistrokes.size(); j++) {
			Templates[j] = new Template(name, useLimitedRotationInvariance, unistrokes.get(j), cPoints);
		}
		
	}
	
	/**
	 * Generates an ArrayList of all possible permutations of the array {0, 1, 2, 3, ... , n}
	 * @param n
	 * @param order
	 * @param orders
	 */
	private void HeapPermute(int n, int[] order, ArrayList<OrderPermutation> orders) {
		if(n == 1) {
			orders.add(new OrderPermutation(Arrays.copyOf(order, order.length)));
		} else {
			for(int i = 0; i < n; i++) {
				HeapPermute(n - 1, order, orders);
				if(n%2 == 1) //swap 0, n-1
				{
					int temp = order[0];
					order[0] = order[n-1];
					order[n-1] = temp;
				}
				else // swap i, n-1 
				{
					int temp = order[i];
					order[i] = order[n-1];
					order[n-1] = temp;
				}
			}
		}
		
	}
	
	/**
	 * Generates every template that can be produced with the given strokes (as a unistroke) in terms of:
	 * 1) The order of the strokes
	 * 2) Whether the strokes are made forwards or backwards
	 * @param strokes
	 * @param orders
	 * @return A 2 dimensional arraylist of all possible unistrokes
	 */
	ArrayList<ArrayList<Point>> MakeUnistrokes(Point[][] strokes, ArrayList<OrderPermutation> orders)
	{
		ArrayList<ArrayList<Point>> unistrokes = new ArrayList<ArrayList<Point>>(); // array of point arrays
		for (int r = 0; r < orders.size(); r++)
		{
			for (int b = 0; b < Math.pow(2, orders.get(r).getOrder().length); b++) // use b's bits for directions
			{
				ArrayList<Point> unistroke = new ArrayList<Point>(); // array of points
				for (int i = 0; i < orders.get(r).getOrder().length; i++)
				{
					Point[] pts;
					if (((b >> i) & 1) == 1) {  // is b's bit at index i on?
						Point[] copy = strokes[orders.get(r).getOrder()[i]];
						List<Point> list = Arrays.asList(copy);
						Collections.reverse(list);
						pts = (Point[]) list.toArray(); // copy and reverse
					} else {
						Point[] copy = strokes[orders.get(r).getOrder()[i]];
						copy = Arrays.copyOf(copy, copy.length);
						pts = copy;
					}
					for (int p = 0; p < pts.length; p++) {
						unistroke.add(pts[p]); // append points
					}
				}
				unistrokes.add(unistroke); // add one unistroke to set
			}
		}
		return unistrokes;
	}
	
}
