package edu.wpi.aipaint.shapedetector;

import java.util.ArrayList;

/**
 * $N implementation - http://depts.washington.edu/aimgroup/proj/dollar/ndollar.html
 * Slightly modified for control point usage
 * 
 * @author amejia
 *
 */
public class Recognizer {
	ArrayList<MultiStroke> Multistrokes;
	
	static final int MultiStroke = 16;
	static final int NumPoints = 96;
	static final float SquareSize = 250.0f;
	static final float OneDThreshold = 0.25f;
	static final Point Origin = new Point(0, 0);
	static final int StartAngleIndex = (NumPoints / 8); // eighth of gesture length	
	static final float Diagonal = (float)Math.sqrt(SquareSize * SquareSize + SquareSize * SquareSize);
	static final float HalfDiagonal = Diagonal * 0.5f;
	static final float AngleRange = (float)Math.toRadians(45.0);
	static final float AnglePrecision = (float)Math.toRadians(2.0);
	static final float Phi = 0.5f * (-1.0f + (float)Math.sqrt(5.0));   // Golden Ratio
	static final float AngleSimilarityThreshold = (float)Math.toRadians(30.0);

	public Recognizer(boolean useLimitedRotationInvariance) {		
		this.Multistrokes = new ArrayList<MultiStroke>();
		
		Point[][] movePts1 = 
		{{new Point(0, 0), new Point(20, 0)},
		 {new Point(15, 3), new Point(20, 0), new Point(15, -3)}
		};
		
		Point[][] movePts2 = 
		{{new Point(0, 0), new Point(20, 0)},
		 {new Point(19, 2), new Point(20, 0), new Point(19, -2)}
		};
		
		Point[][] movePts3 = 
		{{new Point(0, 0), new Point(20, 0)},
		 {new Point(18, 4), new Point(20, 0), new Point(18, -4)}
		};
		
		Point[][] movePts4 = 
		{{new Point(0, 0), new Point(20, 0)},
		 {new Point(15, 1), new Point(20, 0), new Point(15, -1)}
		};
		
		Point[] moveCPts = {new Point(0, 0), new Point(20, 0)};
		
		Point[][] distancePts1 = 
		{{new Point(0, 3), new Point(0, -3)},
		 {new Point(0, 0), new Point(20, 0)},
		 //{new Point(10, 10), new Point(10, -10)},
		 {new Point(18, 5), new Point(20, 0), new Point(18, -5)}
		};
		
		Point[][] distancePts2 = 
		{{new Point(0, 3), new Point(0, -3)},
		 {new Point(0, 0), new Point(20, 0)},
		 //{new Point(10, 10), new Point(10, -10)},
		 {new Point(19, 2), new Point(20, 0), new Point(19, -2)}
		};
		
		Point[][] distancePts3 = 
		{{new Point(0, 3), new Point(0, -3)},
		 {new Point(0, 0), new Point(20, 0)},
		 //{new Point(10, 10), new Point(10, -10)},
		 {new Point(18, 4), new Point(20, 0), new Point(18, -4)}
		};
		
		Point[][] distancePts4 = 
		{{new Point(0, 3), new Point(0, -3)},
		 {new Point(0, 0), new Point(20, 0)},
		 //{new Point(10, 10), new Point(10, -10)},
		 {new Point(15, 1), new Point(20, 0), new Point(15, -1)}
		};
		
		Point[][] distancePts5 = 
		{{new Point(0, 3), new Point(0, -3)},
		 {new Point(0, 0), new Point(20, 0)},
		 //{new Point(5, 10), new Point(5, -10)},
		 {new Point(18, 5), new Point(20, 0), new Point(18, -5)}
		};
		
		Point[][] distancePts6 = 
		{{new Point(0, 3), new Point(0, -3)},
		 {new Point(0, 0), new Point(20, 0)},
		 //{new Point(5, 10), new Point(5, -10)},
		 {new Point(19, 2), new Point(20, 0), new Point(19, -2)}
		};
		
		Point[][] distancePts7 = 
		{{new Point(0, 3), new Point(0, -3)},
		 {new Point(0, 0), new Point(20, 0)},
		 //{new Point(5, 10), new Point(5, -10)},
		 {new Point(18, 4), new Point(20, 0), new Point(18, -4)}
		};
		
		Point[][] distancePts8 = 
		{{new Point(0, 3), new Point(0, -3)},
		 {new Point(0, 0), new Point(20, 0)},
		 //{new Point(5, 10), new Point(5, -10)},
		 {new Point(15, 1), new Point(20, 0), new Point(15, -1)}
		};
		
		Point[][] distancePts9 = 
		{{new Point(0, 3), new Point(0, -3)},
		 {new Point(0, 0), new Point(20, 0)},
		 //{new Point(15, 10), new Point(15, -10)},
		 {new Point(18, 5), new Point(20, 0), new Point(18, -5)}
		};
		
		Point[][] distancePts10 = 
		{{new Point(0, 3), new Point(0, -3)},
		 {new Point(0, 0), new Point(20, 0)},
		// {new Point(15, 10), new Point(15, -10)},
		 {new Point(19, 2), new Point(20, 0), new Point(19, -2)}
		};
		
		Point[][] distancePts11 = 
		{{new Point(0, 3), new Point(0, -3)},
		 {new Point(0, 0), new Point(20, 0)},
		 //{new Point(15, 10), new Point(15, -10)},
		 {new Point(18, 4), new Point(20, 0), new Point(18, -4)}
		};
		
		Point[][] distancePts12 = 
		{{new Point(0, 3), new Point(0, -3)},
		 {new Point(0, 0), new Point(20, 0)},
		 //{new Point(15, 10), new Point(15, -10)},
		 {new Point(15, 1), new Point(20, 0), new Point(15, -1)}
		};
		
		Point[] distanceCPts = {new Point(0, 0), new Point(20, 0)};
		
		Point[][] variablePts = new Point[2][];
		variablePts[0] = new Point[360];
		for(int i = 0; i < 360; i++) {
			variablePts[0][i] = new Point((float)Math.cos(Math.toRadians(i)) * 8.0f, (float)Math.sin(Math.toRadians(i)) * 8.0f);
		}
		variablePts[1] = new Point[2];
		variablePts[1][0] = new Point(-20, 0);
		variablePts[1][1] = new Point(0, 0);
		Point[] variableCPts = {new Point(-20, 0), new Point(0, 0)};
		
		Multistrokes.add(new MultiStroke("DistanceTo", useLimitedRotationInvariance, distancePts1, distanceCPts));
		Multistrokes.add(new MultiStroke("DistanceTo", useLimitedRotationInvariance, distancePts2, distanceCPts));
		Multistrokes.add(new MultiStroke("DistanceTo", useLimitedRotationInvariance, distancePts3, distanceCPts));
		Multistrokes.add(new MultiStroke("DistanceTo", useLimitedRotationInvariance, distancePts4, distanceCPts));
		Multistrokes.add(new MultiStroke("DistanceTo", useLimitedRotationInvariance, distancePts5, distanceCPts));
		Multistrokes.add(new MultiStroke("DistanceTo", useLimitedRotationInvariance, distancePts6, distanceCPts));
		Multistrokes.add(new MultiStroke("DistanceTo", useLimitedRotationInvariance, distancePts7, distanceCPts));
		Multistrokes.add(new MultiStroke("DistanceTo", useLimitedRotationInvariance, distancePts8, distanceCPts));
		Multistrokes.add(new MultiStroke("DistanceTo", useLimitedRotationInvariance, distancePts9, distanceCPts));
		Multistrokes.add(new MultiStroke("DistanceTo", useLimitedRotationInvariance, distancePts10, distanceCPts));
		Multistrokes.add(new MultiStroke("DistanceTo", useLimitedRotationInvariance, distancePts11, distanceCPts));
		Multistrokes.add(new MultiStroke("DistanceTo", useLimitedRotationInvariance, distancePts12, distanceCPts));
		
		//Point[][] areaPts = 
		//{{new Point(0, 0), new Point(10, 0), new Point(10, 10), new Point(0, 10)}};
		
		//Point[] areaCPts = {new Point(0, 0), new Point(10, 10)};
		
		//Multistrokes.add(new MultiStroke("Area", useLimitedRotationInvariance, areaPts, areaCPts));
		
		Multistrokes.add(new MultiStroke("MoveTo", useLimitedRotationInvariance, movePts1, moveCPts));
		Multistrokes.add(new MultiStroke("MoveTo", useLimitedRotationInvariance, movePts2, moveCPts));
		Multistrokes.add(new MultiStroke("MoveTo", useLimitedRotationInvariance, movePts3, moveCPts));
		Multistrokes.add(new MultiStroke("MoveTo", useLimitedRotationInvariance, movePts4, moveCPts));
		
		Multistrokes.add(new MultiStroke("Variable", useLimitedRotationInvariance, variablePts, variableCPts));
	}	
	
	public ArrayList<Result> recognize(Point[][] strokes, boolean matchOnlyIfSameNumberOfStrokes, boolean useLimitedRotationInvariance) {
		ArrayList<Point> points = CombineStrokes(strokes); // make one connected unistroke from the given strokes		
		
		//Transform our strokes the same way we would a stored multistroke in order to normalize the shape
		points = Resample(points, NumPoints);
		ArrayList<Point> pointsCopy = new ArrayList<Point>(points);
		float radians = IndicativeAngle(points);
		points = RotateBy(points, -radians);
		points = ScaleDimTo(points, SquareSize, OneDThreshold);
		if (useLimitedRotationInvariance) points = RotateBy(points, +radians);
		points = TranslateTo(points, Origin);
		Point startv = CalcStartUnitVector(points, StartAngleIndex);
		
		ArrayList<Result> results = new ArrayList<Result>();
		for (int i = 0; i < this.Multistrokes.size(); i++) // compare the normalized shape with each multistroke...
		{
			if (!matchOnlyIfSameNumberOfStrokes || strokes.length == this.Multistrokes.get(i).NumStrokes) // optional -- only attempt match when number of strokes is same
			{
				for (int j = 0; j < this.Multistrokes.get(i).Templates.length; j++) // compare teh normalized shape with each unistroke permutation
				{	
					if (AngleBetweenUnitVectors(startv, this.Multistrokes.get(i).Templates[j].getStartUnitVector()) <= AngleSimilarityThreshold)
					{	
						float d = DistanceAtBestAngle(points, this.Multistrokes.get(i).Templates[j], -AngleRange, +AngleRange, AnglePrecision); // iterative start
						
						int[] controlPointIndices = this.Multistrokes.get(i).Templates[j].getControlPointIndices();
						Point[] controlPoints = new Point[controlPointIndices.length];
						for(int pointIndex = 0; pointIndex < controlPointIndices.length; pointIndex++) {
							controlPoints[pointIndex] = pointsCopy.get(controlPointIndices[pointIndex]);
						}
						float resultValue = 1.0f - (d / HalfDiagonal);
						Result result = new Result(this.Multistrokes.get(i).Name, resultValue, controlPoints);
						if(results.size() == 0) {
							results.add(result);
						} else {
							int indexOfResult = getIndexOfResultWithMultistrokeName(Multistrokes.get(i).Name, results);
							boolean insertInResultsList = true;
							if(indexOfResult >= 0) {
								if(resultValue > results.get(indexOfResult).getScore()) {
									results.remove(indexOfResult);
								} else {
									insertInResultsList = false;
								}
							}
							if(insertInResultsList) {
								int currentIndex = 0;
								while(currentIndex < results.size() && results.get(currentIndex).getScore() > resultValue) {
									currentIndex++;
								}
								if(currentIndex < results.size())
									results.add(currentIndex, result);
								else
									results.add(result);
							}
						}
					}
				}
			}
		}
		return results;
	}
	
	private int getIndexOfResultWithMultistrokeName(String name, ArrayList<Result> results) {
		int i = 0;
		while(i < results.size() && !results.get(i).getName().equals(name))
			i++;
		if(i >= results.size())
			return -1;
		return i;
	}
	
	/**
	 * Combines strokes into a unistroke
	 * @param strokes
	 * @return
	 */
	private ArrayList<Point> CombineStrokes(Point[][] strokes)
	{
		ArrayList<Point> points = new ArrayList<Point>();
		for (int s = 0; s < strokes.length; s++) {
			for (int p = 0; p < strokes[s].length; p++) {
				points.add(new Point(strokes[s][p].X, strokes[s][p].Y));
			}
		}
		return points;
	}
	
	/**
	 * Resample a shape to give it n points
	 * @param points
	 * @param n
	 * @return
	 */
	public static ArrayList<Point> Resample(ArrayList<Point> points, int n)
	{
		if(points.size() <= 1) {
			return new ArrayList<Point>();
		}
		float I = PathLength(points) / (n - 1); // interval length
		float D = 0.0f;
		ArrayList<Point> newpoints = new ArrayList<Point>();
		newpoints.add(points.get(0));
		int i = 1;
		while(i < points.size())
		{
			float d = Distance(points.get(i - 1), points.get(i));
			if ((D + d) >= I)
			{
				float qx = points.get(i - 1).X + ((I - D) / d) * (points.get(i).X - points.get(i - 1).X);
				float qy = points.get(i - 1).Y + ((I - D) / d) * (points.get(i).Y - points.get(i - 1).Y);
				Point q = new Point(qx, qy);
				newpoints.add(q); // append new point 'q'
				points.add(i, q); // insert 'q' at position i in points s.t. 'q' will be the next i
				D = 0.0f;
			}
			else D += d;
			i++;
		}
		// somtimes we fall a rounding-error short of adding the last point, so add it if so
		if (newpoints.size() == n - 1)
		{
			newpoints.add(new Point(points.get(points.size() - 1).X, points.get(points.size()- 1).Y));
		}

		if(newpoints.size() != n)
			throw new RuntimeException();
		
		return newpoints;
	}
	
	/**
	 * Returns the angle from the first point in the unistroke to the centroid
	 * @param points
	 * @return
	 */
	public static float IndicativeAngle(ArrayList<Point> points)
	{
		Point c = Centroid(points);
		return (float)Math.atan2(c.Y - points.get(0).Y, c.X - points.get(0).X);
	}
	
	
	
	static float PathLength(ArrayList<Point> points) // length traversed by a point path
	{
		float d = 0.0f;
		for (int i = 1; i < points.size(); i++)
			d += Distance(points.get(i - 1), points.get(i));
		return d;
	}	
	
	static float Distance(Point p1, Point p2) // distance between two points
	{
		float dx = p2.X - p1.X;
		float dy = p2.Y - p1.Y;
		return (float)Math.sqrt(dx * dx + dy * dy);
	}
	
	static float DistanceSquared(Point p1, Point p2) // squared distance between points for faster calculations
	{
		float dx = p2.X - p1.X;
		float dy = p2.Y - p1.Y;
		return (dx * dx + dy * dy);
	}
	
	static Point Centroid(ArrayList<Point> points)
	{
		float x = 0.0f;
		float y = 0.0f;
		for (int i = 0; i < points.size(); i++)
		{
			x += points.get(i).X;
			y += points.get(i).Y;
		}
		x /= points.size();
		y /= points.size();
		return new Point(x, y);
	}	
	
	static ArrayList<Point> TranslateTo(ArrayList<Point> points, Point pt) // translates points' centroid
	{
		Point c = Recognizer.Centroid(points);
		ArrayList<Point> newpoints = new ArrayList<Point>();
		for (int i = 0; i < points.size(); i++)
		{
			float qx = points.get(i).X + pt.X - c.X;
			float qy = points.get(i).Y + pt.Y - c.Y;
			newpoints.add(new Point(qx, qy));
		}
		return newpoints;
	}
	
	static ArrayList<Point> TranslateTo(ArrayList<Point> points, Point pt, Point translateBy) {
		ArrayList<Point> newpoints = new ArrayList<Point>();
		for (int i = 0; i < points.size(); i++)
		{
			float qx = points.get(i).X + pt.X - translateBy.X;
			float qy = points.get(i).Y + pt.Y - translateBy.Y;
			newpoints.add(new Point(qx, qy));
		}
		return newpoints;
	}
	
	static ArrayList<Point> RotateBy(ArrayList<Point> points, float radians) // rotates points around centroid
	{
		Point c = Recognizer.Centroid(points);
		float cos = (float)Math.cos(radians);
		float sin = (float)Math.sin(radians);
		
		ArrayList<Point> newpoints = new ArrayList<Point>();
		for (int i = 0; i < points.size(); i++)
		{
			float qx = (points.get(i).X - c.X) * cos - (points.get(i).Y - c.Y) * sin + c.X;
			float qy = (points.get(i).X - c.X) * sin + (points.get(i).Y - c.Y) * cos + c.Y;
			newpoints.add(new Point(qx, qy));
		}
		return newpoints;
	}
	
	static ArrayList<Point> ScaleDimTo(ArrayList<Point> points, float size, float oneDratio) // scales bbox uniformly for 1D, non-uniformly for 2D
	{
		Rectangle B = BoundingBox(points);
		boolean uniformly = Math.min(B.Width / B.Height, B.Height / B.Width) <= oneDratio; // 1D or 2D gesture test
		ArrayList<Point> newpoints = new ArrayList<Point>();
		for (int i = 0; i < points.size(); i++)
		{
			float qx = uniformly ? points.get(i).X * (size / Math.max(B.Width, B.Height)) : points.get(i).X * (size / B.Width);
			float qy = uniformly ? points.get(i).Y * (size / Math.max(B.Width, B.Height)) : points.get(i).Y * (size / B.Height);
			newpoints.add(new Point(qx, qy));
		}
		return newpoints;
	}
	
	
	static Rectangle BoundingBox(ArrayList<Point> points)
	{
		float minX = Float.MAX_VALUE;
		float maxX = Float.MIN_VALUE;
		float minY = Float.MAX_VALUE;
		float maxY = Float.MIN_VALUE;
		for (int i = 0; i < points.size(); i++)
		{
			if (points.get(i).X < minX)
				minX = points.get(i).X;
			if (points.get(i).X > maxX)
				maxX = points.get(i).X;
			if (points.get(i).Y < minY)
				minY = points.get(i).Y;
			if (points.get(i).Y > maxY)
				maxY = points.get(i).Y;
		}
		return new Rectangle(minX, minY, maxX - minX, maxY - minY);
	}
	
	static Point CalcStartUnitVector(ArrayList<Point> points, int index) // start angle from points[0] to points[index] normalized as a unit vector
	{
		Point v = new Point(points.get(index).X - points.get(0).X, points.get(index).Y - points.get(0).Y);
		float len = (float)Math.sqrt(v.X * v.X + v.Y * v.Y);
		return new Point(v.X / len, v.Y / len);
	}
	
	float AngleBetweenUnitVectors(Point v1, Point v2) // gives acute angle between unit vectors from (0,0) to v1, and (0,0) to v2
	{
		float n = (v1.X * v2.X + v1.Y * v2.Y);
		if (n < -1.0 || n > +1.0)
			n = Round(n, 5); // fix JS rounding bug that can occur so that -1<=n<=+1
		return (float)Math.acos(n); // arc cosine of the vector dot product
	}
	
	float Round(float n, int d) { d = (int)Math.pow(10,d); return Math.round(n*d)/d; } // round 'n' to 'd' decimals
	
	float DistanceAtBestAngle(ArrayList<Point> points, Template T, float a, float b, float threshold)
	{
		float x1 = Phi * a + (1.0f - Phi) * b;
		float f1 = DistanceAtAngle(points, T, x1);
		float x2 = (1.0f - Phi) * a + Phi * b;
		float f2 = DistanceAtAngle(points, T, x2);
		while (Math.abs(b - a) > threshold)
		{
			if (f1 < f2)
			{
				b = x2;
				x2 = x1;
				f2 = f1;
				x1 = Phi * a + (1.0f - Phi) * b;
				f1 = DistanceAtAngle(points, T, x1);
			}
			else
			{
				a = x1;
				x1 = x2;
				f1 = f2;
				x2 = (1.0f - Phi) * a + Phi * b;
				f2 = DistanceAtAngle(points, T, x2);
			}
		}
		return Math.min(f1, f2);
	}
	
	float DistanceAtAngle(ArrayList<Point> points, Template T, float radians)
	{
		ArrayList<Point> newpoints = RotateBy(points, radians);
		return PathDistance(newpoints, T.getPoints());
	}	
	
	float PathDistance(ArrayList<Point> pts1, ArrayList<Point> pts2) // average distance between corresponding points in two paths
	{
		float d = 0.0f;
		for (int i = 0; i < pts1.size(); i++) // assumes pts1.length == pts2.length
			d += Distance(pts1.get(i), pts2.get(i));
		return d / pts1.size();
	}
	
	//
	// add/delete new multistrokes
	//
	public int AddMultistroke(String name, boolean useLimitedRotationInvariance, Point[][] strokes, Point[] controlPoints)
	{
		Multistrokes.add(new MultiStroke(name, useLimitedRotationInvariance, strokes, controlPoints));
		int num = 0;
		for (int i = 0; i < this.Multistrokes.size(); i++)
		{
			if (this.Multistrokes.get(i).Name == name)
				num++;
		}
		return num;
	}
}