package edu.wpi.aipaint.shapedetector;

import java.util.ArrayList;

/**
 * $N implementation - http://depts.washington.edu/aimgroup/proj/dollar/ndollar.html
 * Slightly modified for control point usage
 * @author amejia
 *
 */
public class Template {
	String name;
	ArrayList<Point> myPoints;
	Point startUnitVector;
	int[] controlPointIndices;
	
	public Template(String name, boolean useLimitedRotationInvariance, ArrayList<Point> points, ArrayList<Point> cPoints) {
		this.name = name;
		this.myPoints = Recognizer.Resample(points, Recognizer.NumPoints);
		//After resampling, we assign our desired control points to the points closest to them.
		findControlPointIndices(cPoints);
		float radians = Recognizer.IndicativeAngle(myPoints);
		myPoints = Recognizer.RotateBy(myPoints, -radians);
		myPoints = Recognizer.ScaleDimTo(myPoints, Recognizer.SquareSize, Recognizer.OneDThreshold);
		if(useLimitedRotationInvariance) {
			myPoints = Recognizer.RotateBy(myPoints, radians);
		}
		myPoints = Recognizer.TranslateTo(myPoints, Recognizer.Origin);
		this.startUnitVector = Recognizer.CalcStartUnitVector(myPoints, Recognizer.StartAngleIndex);
	}	
	
	//Finds the indices of the points in the template closest to the given control points
	private void findControlPointIndices(ArrayList<Point> cPoints) {
		controlPointIndices = new int[cPoints.size()];

		for(int cIndex = 0; cIndex < cPoints.size(); cIndex++) {
			Point control = cPoints.get(cIndex);
			float bestDistance = Float.MAX_VALUE;
			for(int p = 0; p < myPoints.size(); p++) {
				float distSquared = Recognizer.DistanceSquared(control, myPoints.get(p));
				if(distSquared < bestDistance) {
					bestDistance = distSquared;
					controlPointIndices[cIndex] = p;
				}
			}
		}
	}
	
	public Point getStartUnitVector() {
		return startUnitVector;
	}
	
	public ArrayList<Point> getPoints() {
		return myPoints;
	}
	
	public int[] getControlPointIndices() {
		return controlPointIndices;
	}
	
	
	
	
}

