package edu.wpi.aipaint.sketcho;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import edu.wpi.aipaint.ai.BTNominalDecisionNode;
import edu.wpi.aipaint.attributes.AreaAttribute;
import edu.wpi.aipaint.attributes.Attribute;
import edu.wpi.aipaint.directo.AttachChildDirection;
import edu.wpi.aipaint.pipeline.Direction;
import edu.wpi.aipaint.util.Line;
import edu.wpi.aipaint.util.Vector2;
import edu.wpi.aipaint.util.Line.LineFonts;
import edu.wpi.aipaint.values.BooleanWrapper;
import edu.wpi.aipaint.values.Value;
import edu.wpi.aipaint.worldspacepoint.WorldSpacePoint;

public class AreaConditionalStatement extends SketchoConditionalStatement {
	ArrayList<SketchoStatement> children;
	SketchoId childId1, childId2;
	
	public AreaConditionalStatement() {
		children = new ArrayList<SketchoStatement>();
		childId1 = new SketchoId();
		childId2 = new SketchoId();
	}
	
	public LineFonts getChildFont(int childIndex) {
		return LineFonts.solid;
	}

	@Override
	public Iterable<Line> getDrawLines() {
		Line[] lines = new Line[4];
		lines[0] = new Line(controlPoint1.getPoint(), new Vector2(controlPoint1.getX(), controlPoint2.getY()), mainColor);
		lines[1] = new Line(new Vector2(controlPoint1.getX(), controlPoint2.getY()), controlPoint2.getPoint(), mainColor);
		lines[2] = new Line(controlPoint2.getPoint(), new Vector2(controlPoint2.getX(), controlPoint1.getY()), mainColor); 
		lines[3] = new Line(new Vector2(controlPoint2.getX(), controlPoint1.getY()), controlPoint1.getPoint(), mainColor);
		return Arrays.asList(lines);
	}
	
	public Iterable<DrawableString> getDrawableWords() {
		ArrayList<DrawableString> wordsList = new ArrayList<DrawableString>();
		Vector2 insidePoint;
		Color insideColor;
		String insideString;
		
		insidePoint = controlPoint1.getPoint().add(controlPoint2.getPoint()).multiply(.5f);
		insideString = childId1.getIdString();
		if(children.size() > 0) {			
			insideColor = mainColor;
		} else {
			insideColor = Color.red;
		}
		
		wordsList.add(new DrawableString(insideString, insidePoint, insideColor));
		
		if(children.size() == 0)
			return wordsList;
		
		Vector2 outsidePoint;
		Color outsideColor;
		String outsideString = childId2.getIdString();
		
		outsidePoint = controlPoint1.getPoint().add(new Vector2(-5, 0));
		if(children.size() > 1) {
			outsideColor = mainColor;
		} else {
			outsideColor = Color.red;
		}
		
		wordsList.add(new DrawableString(outsideString, outsidePoint, outsideColor));
		
		
		return wordsList;
	}

	@Override
	public void doubleClick(Vector2 worldSpacePoint) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public int numberChildrenNeeded() {
		return 2;
	}

	@Override
	public List<Direction> translate() {
		List<Direction> translation = new LinkedList<Direction>();
		
		List<Value> decisionSlots = new LinkedList<Value>();
		decisionSlots.add(new BooleanWrapper(true));
		decisionSlots.add(new BooleanWrapper(false));
		
		// TODO: This Statement makes absolutely no sense.
		// It tests whether one of its control points is within the rectangle
		// specified by its control points; it should instead test whether
		// some other Attribute is within the rectangle specified by its
		// control points.
		
		Attribute point1 = controlPoint1.translateToAttribute();
		
		BTNominalDecisionNode areaDecision = new BTNominalDecisionNode(
				null,
				new AreaAttribute(point1, controlPoint1.getPoint(), controlPoint2.getPoint()),
				decisionSlots);
		
		AttachChildDirection addAreaDecision = new AttachChildDirection(areaDecision);
		
		translation.add(addAreaDecision);
		
		return translation;
	}

	@Override
	public Iterable<WorldSpacePoint> getGeneralizableControlPoints() {
		LinkedList<WorldSpacePoint> controlPoints = new LinkedList<WorldSpacePoint>();
		controlPoints.add(controlPoint1);
		controlPoints.add(controlPoint2);
		return controlPoints;
	}

	@Override
	public void setGeneralizableControlPoint(WorldSpacePoint point, int index) {
		if (index == 0)
			controlPoint1 = point;
		else
			controlPoint2 = point;
	}
}
