package edu.wpi.aipaint.sketcho;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import edu.wpi.aipaint.ai.BTContinuousDecisionNode;
import edu.wpi.aipaint.attributes.Attribute;
import edu.wpi.aipaint.attributes.DistanceAttribute;
import edu.wpi.aipaint.directo.AttachChildDirection;
import edu.wpi.aipaint.pipeline.Direction;
import edu.wpi.aipaint.util.Line;
import edu.wpi.aipaint.util.Line.LineFonts;
import edu.wpi.aipaint.util.Vector2;
import edu.wpi.aipaint.values.ContinuousValue;
import edu.wpi.aipaint.values.FloatWrapper;
import edu.wpi.aipaint.worldspacepoint.WorldSpacePoint;

public class DistanceConditionalStatement extends SketchoConditionalStatement {
	ArrayList<SketchoStatement> children;
	private float threshold;
	private Selections selectedPart;
	
	private final float MIN_DISTANCE_TO_SELECT_PART = 15;
	
	public DistanceConditionalStatement() {
		children = new ArrayList<SketchoStatement>();
	}
	
	@Override
	public void setControlPoint2(WorldSpacePoint wsp) {
		controlPoint2 = wsp;
		Vector2 arrowBody = controlPoint1.getPoint().subtract(controlPoint2.getPoint());
		threshold = arrowBody.getMagnitude()/2.0f;
	}
	
	@Override
	public Iterable<Line> getDrawLines() {
		Vector2 cp1 = controlPoint1.getPoint();
		Vector2 cp2 = controlPoint2.getPoint();
		
		if(Math.abs(cp1.getX() - cp2.getX()) < .01 && Math.abs(cp1.getY() - cp2.getY()) < .01)
			return new ArrayList<Line>();
		float dist = translator.screenSpaceToWorldSpace(new Vector2(20, 0)).subtract(translator.screenSpaceToWorldSpace(new Vector2(0, 0))).getMagnitude();
		
		
		
		Line[] drawLine;
		
		 
		drawLine = new Line[6];
		
		//drawLine[0] = new Line(cp1, cp2, mainColor);
		
		Vector2 arrowHead1 = cp1.rotate(cp2, 45);		
		arrowHead1 = arrowHead1.subtract(cp2);
		arrowHead1 = arrowHead1.normalize();
		arrowHead1 = arrowHead1.multiply(dist);
		arrowHead1 = arrowHead1.add(cp2);
		drawLine[1] = new Line(cp2, arrowHead1, mainColor);
		
		Vector2 arrowHead2 = cp1.rotate(cp2, -45);
		arrowHead2 = arrowHead2.subtract(cp2);
		arrowHead2 = arrowHead2.normalize();
		arrowHead2 = arrowHead2.multiply(dist);
		arrowHead2 = arrowHead2.add(cp2);
		drawLine[2] = new Line(cp2, arrowHead2, mainColor);
		
		Vector2 arrowDirection = cp2.subtract(cp1);
		arrowDirection  = arrowDirection.normalize();
		Vector2 thresholdMark = arrowDirection.multiply(threshold);
		Vector2 halfLength = arrowDirection.multiply(dist/2);
		Vector2 separatorPoint1 = thresholdMark.add(halfLength);
		Vector2 separatorPoint2 = thresholdMark.add(halfLength);
		separatorPoint1 = separatorPoint1.rotate(thresholdMark, 90);
		separatorPoint2 = separatorPoint2.rotate(thresholdMark, -90);
	
		Vector2 basePoint1 = separatorPoint1.subtract(thresholdMark);
		Vector2 basePoint2 = separatorPoint2.subtract(thresholdMark);
	
		separatorPoint1 = separatorPoint1.add(cp1);
		separatorPoint2 = separatorPoint2.add(cp1);
		basePoint1 = basePoint1.add(cp1);
		basePoint2 = basePoint2.add(cp1);
		Vector2 midPoint = thresholdMark.add(cp1);
		drawLine[0] = new Line(cp1, midPoint, mainColor);
		drawLine[5] = new Line(midPoint, cp2, mainColor, LineFonts.dotted);
		drawLine[3] = new Line(separatorPoint1, separatorPoint2, mainColor);
		drawLine[4] = new Line(basePoint1, basePoint2, mainColor);
		
		
		return Arrays.asList(drawLine);
	}
	
	public LineFonts getChildFont(int childIndex) {
		if(childIndex == 0)
			return LineFonts.solid;
		else
			return LineFonts.dotted;
	}
	
	public Iterable<DrawableString> getDrawableWords() {
		ArrayList<DrawableString> wordsList = new ArrayList<DrawableString>(); 
		
		Vector2 shortPoint;
		/*
		Color shortColor;
		
		if(children.size() > 0)
			shortColor = mainColor;
		else
			shortColor = Color.red;
		*/
		
		shortPoint = controlPoint1.getPoint().add(controlPoint2.getPoint()).multiply(.5f);
		shortPoint = controlPoint1.getPoint().add(shortPoint).multiply(.5f);
		
		
		if(children.size() == 0)
			return wordsList;
		
		Vector2 longPoint;
		/*
		Color longColor;
		
		if(children.size() > 1)
			longColor = mainColor;
		else
			longColor = Color.red;
		*/
		
		longPoint = controlPoint1.getPoint().add(controlPoint2.getPoint()).multiply(.5f);
		longPoint = controlPoint2.getPoint().add(longPoint).multiply(.5f);
		
		return wordsList;
	}

	public void select(Vector2 e) {
		super.select(e);
		float minDistance = Float.MAX_VALUE;
		int index = 0;
		
		Iterable<Line> lines = getDrawLines();
		int i = 0;
		for(Line l : lines) {
			float distance = l.distanceToPoint(new Vector2(e.getX(), e.getY()));
			if (distance < minDistance) {
				minDistance = distance;
				index = i;
			}
			i++;
		}		
		
		if(minDistance >= MIN_DISTANCE_TO_SELECT_PART)
			index = 0;
		
		switch(index) {
		case 0:	//body
			float fullLength = controlPoint1.getPoint().distanceTo(controlPoint2.getPoint());
			
			//check if the click is close to the tail
			if(controlPoint1.getPoint().distanceTo(new Vector2(e.getX(), e.getY())) < fullLength/5.0f)
				selectedPart = Selections.tail;
			else if (controlPoint2.getPoint().distanceTo(new Vector2(e.getX(), e.getY())) < fullLength / 5.0f)
				selectedPart = Selections.head;
			else
				selectedPart = Selections.body;
			break;
		case 1: //arrowHead
		case 2: //arrowHead
			selectedPart = Selections.head;
			break;
		case 3: //Threshold
			selectedPart = Selections.threshold;
			break;
		case 4: //tail
			selectedPart = Selections.tail;
			break;
		}
	}
	
	@Override
	public void deSelect() {
		selectedPart = null;
		setColor(UNSELECTION_COLOR);
	}

	private enum Selections{
		tail, head, threshold, body
	}

	@Override
	public void doubleClick(Vector2 worldSpacePoint) { }

	@Override
	public void mouseDrag(Vector2 offset, WorldSpacePoint newPoint) {
		if(selectedPart == Selections.threshold) {
			Vector2 bodyVector = controlPoint2.getPoint().subtract(controlPoint1.getPoint());
			float projectedOffset = bodyVector.dot(offset) / bodyVector.getMagnitude();
			threshold += projectedOffset;
			threshold = Math.max(0, Math.min(bodyVector.getMagnitude(), threshold));
		} else {
			super.mouseDrag(offset, newPoint);
		}
	}
	
	public void setThreshold(float threshold)
	{
		this.threshold = threshold;
	}

	@Override
	public int numberChildrenNeeded() {
		return 2;
	}

	@Override
	public List<Direction> translate() {
		List<Direction> translation = new LinkedList<Direction>();
		
		List<ContinuousValue> decisionSlots = new LinkedList<ContinuousValue>();
		decisionSlots.add(new FloatWrapper(threshold));
		decisionSlots.add(FloatWrapper.MAX_VALUE);
		
		Attribute point1 = controlPoint1.translateToAttribute();
		
		Attribute point2 = controlPoint2.translateToAttribute();
		
		DistanceAttribute distance = new DistanceAttribute(point1, point2);
		distance.setThreshold(threshold);
		
		BTContinuousDecisionNode distanceDecision = 
			new BTContinuousDecisionNode(
				distance, null, decisionSlots, 
				FloatWrapper.ZERO, FloatWrapper.MAX_VALUE);
		
		AttachChildDirection addDistanceDecision = 
			new AttachChildDirection(distanceDecision);
		
		translation.add(addDistanceDecision);
		
		return translation;
	}
	
	@Override
	public Iterable<WorldSpacePoint> getGeneralizableControlPoints() {
		LinkedList<WorldSpacePoint> controlPoints = new LinkedList<WorldSpacePoint>();
		controlPoints.add(controlPoint1);
		controlPoints.add(controlPoint2);
		return controlPoints;
	}

	@Override
	public void setGeneralizableControlPoint(WorldSpacePoint point, int index) {
		if (index == 0)
			controlPoint1 = point;
		else
			controlPoint2 = point;
	}
}
