package edu.wpi.aipaint.sketcho;

import java.util.LinkedList;

import edu.wpi.aipaint.attributes.Attribute;
import edu.wpi.aipaint.main.GameKeeper;
import edu.wpi.aipaint.main.Group;
import edu.wpi.aipaint.main.Poseable;
import edu.wpi.aipaint.main.WorldState;
import edu.wpi.aipaint.values.Value;


/**
 * Generalizes from a Poseable to an Attribute representing a Group of Poseables.
 */
public class Generalizer {
	private HeuristicFactory heuristicFactory;
	
	public Generalizer(HeuristicFactory heuristicFactory)
	{
		this.heuristicFactory = heuristicFactory;
	}
	
	public Attribute generalize(Poseable specialPoseable)
	{
		LinkedList<Attribute> potentialGeneralizations = new LinkedList<Attribute>();
		
		Iterable<Group> groups = specialPoseable.getGroups();
		for (Group group : groups)
		{
			// Ask an abstract HeuristicFactory for an Iterable of Attributes
			// that can generalize this particular Group.
			Iterable<Attribute> generalizingAttributes = 
				heuristicFactory.getGeneralizingAttributes(group);
			
			// Iterate over these Attributes.
			for (Attribute attribute : generalizingAttributes)
			{
				WorldState worldState = GameKeeper.GetInstance().getCurrentWorldState();
				
				// Compute the generalized Value of this Attribute
				Value generalizedValue = attribute.evaluate(worldState);
				
				// If our specialPoseable matches the generalized Value, 
				// we should add an item to the List of potential generalizations.
				if (worldState.getValue(specialPoseable.getFeature()).equals(generalizedValue))
					potentialGeneralizations.add(attribute);
			}
		}
		
		// TODO: If there is more than one potential generalization,
		// ask the user to decide between them.
		if (potentialGeneralizations.size() > 0)
			return potentialGeneralizations.getFirst();
		
		return null;
	}
}
