package edu.wpi.aipaint.sketcho;

import edu.wpi.aipaint.main.Game;
import edu.wpi.aipaint.util.Vector2;

/**
 * Class for translating from screen space to a given game's world space and 
 * back. 
 * @author amejia
 *
 */
public class ScreenSpaceWorldSpaceTranslator {
	/*
	 * RATIONALE: Sketcho statement's position needs to be defined both in
	 * screen space and in world space. For example,the end points of a distance 
	 * conditional statement need to be drawn in world space (to match the position
	 * of an entity they might be following), but at the same time they are 
	 * interacted with in screen space (for example, dragging the threshold bar of
	 * the aforementioned distance conditional). Thus, they need to be able to do 
	 * these conversions at any time. The Game class isn't passed directly to these
	 * objects because we don't want to cause any couplings with the other parts of
	 * Game's interfaces - just the screen space to world space conversions.
	 */
	private Game game;
	public ScreenSpaceWorldSpaceTranslator(Game game) {
		this.game = game;
	}
	
	public Vector2 screenSpaceToWorldSpace(Vector2 screenSpace) {
		return game.getWorldSpacePoint(screenSpace);
	}
	
	public Vector2 worldSpaceToScreenSpace(Vector2 worldSpace) {
		return game.getScreenSpacePoint(worldSpace);
	}
}
