package edu.wpi.aipaint.sketcho;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import edu.wpi.aipaint.main.Poseable;
import edu.wpi.aipaint.pipeline.Direction;
import edu.wpi.aipaint.util.Line;
import edu.wpi.aipaint.util.Vector2;
import edu.wpi.aipaint.util.Line.LineFonts;
import edu.wpi.aipaint.worldspacepoint.WorldSpacePoint;
import edu.wpi.aipaint.worldspacepoint.WorldSpacePointOffset;

public class SketchoIntermediateVariable extends SketchoStatement {
	
	private final float SCREEN_SPACE_RADIUS = 1.0f;
	
	@Override
	public List<Direction> translate() {
		return null;
	}
 
	@Override
	public void setControlPoint2(WorldSpacePoint wsp) {
		super.setControlPoint2(wsp);
		setControlPoint2Follow();
	}
		
	private void setControlPoint2Follow() {
		if(controlPoint1.holdsPoseable()) {
			Poseable p = controlPoint1.getPoseable();
			Vector2 direction = (controlPoint2.getPoint().subtract(p.getPosition()));
			float distance = direction.getMagnitude();
			if(Float.isInfinite(distance) || Float.isNaN(distance) || distance == 0)
				return;
			float angleBetween = (float)Math.toDegrees(Math.atan2(direction.getY(), direction.getX()) - Math.atan2(p.getOrientation().getY(), p.getOrientation().getX()));
			controlPoint2 = new WorldSpacePointOffset(p, angleBetween, distance);
		}
	}
	
	public LineFonts getChildFont(int childIndex) {
		return LineFonts.solid;
	}
	
	@Override
	public Iterable<WorldSpacePoint> getGeneralizableControlPoints() {
		return new ArrayList<WorldSpacePoint>();
	}

	@Override
	public void setGeneralizableControlPoint(WorldSpacePoint point, int index) {
		// do nothing

	}

	@Override
	public int numberChildrenNeeded() {
		return 0;
	}

	@Override
	public Iterable<Line> getDrawLines() {
		float radius = translator.screenSpaceToWorldSpace(new Vector2(SCREEN_SPACE_RADIUS, 0)).getMagnitude();
		Line[] lines = new Line[361];
		for(int i = 0; i < lines.length; i++) {
			Vector2 start = new Vector2((float)Math.cos(Math.toRadians(i)), (float)Math.sin(Math.toRadians(i))).multiply(radius);
			Vector2 end = new Vector2((float)Math.cos(Math.toRadians(i+1)), (float)Math.sin(Math.toRadians(i+1))).multiply(radius);
			start = start.add(controlPoint2.getPoint());
			end = end.add(controlPoint2.getPoint());
			lines[i] = new Line(start, end, mainColor);
		}
		lines[360] = new Line(controlPoint1.getPoint(), controlPoint2.getPoint(), mainColor);
		return Arrays.asList(lines);
	}

	@Override
	public Iterable<DrawableString> getDrawableWords() {
		return null;
	}

	@Override
	public void doubleClick(Vector2 worldSpacePoint) {
		// TODO Auto-generated method stub

	}
	
	public boolean fitsSyntaxTree() {
		return false;
	}

}
