package edu.wpi.aipaint.sketcho;

import java.awt.Color;

import edu.wpi.aipaint.pipeline.Statement;
import edu.wpi.aipaint.pipeline.TreeNode;
import edu.wpi.aipaint.util.Line;
import edu.wpi.aipaint.util.Line.LineFonts;
import edu.wpi.aipaint.util.Vector2;
import edu.wpi.aipaint.worldspacepoint.WorldSpacePoint;

public abstract class SketchoStatement implements Statement, TreeNode {
	
	protected WorldSpacePoint controlPoint1, controlPoint2;
	protected Color mainColor;
	protected static final Color SELECTION_COLOR = Color.red;
	protected static final Color UNSELECTION_COLOR = Color.white;
	protected ScreenSpaceWorldSpaceTranslator translator;
	protected SketchoId id;
	protected Vector2 mouseClick;
	protected int selectedPoint;
	protected float minDistanceForControlPointSelection;
	protected float MIN_SPACE_POINT_DISTANCE = 10;
	
	public SketchoStatement() {
		mainColor = Color.white;
		minDistanceForControlPointSelection = MIN_SPACE_POINT_DISTANCE;
	}
	
	public void setId(SketchoId id) {
		this.id = id;
	}
	
	public SketchoId getId() {
		return id;
	}
	
	public void setTranslator(ScreenSpaceWorldSpaceTranslator translator) {
		this.translator = translator;
		minDistanceForControlPointSelection = translator.screenSpaceToWorldSpace(new Vector2(0, MIN_SPACE_POINT_DISTANCE)).getMagnitude();
	}
	
	public abstract Iterable<WorldSpacePoint> getGeneralizableControlPoints();
	
	public abstract void setGeneralizableControlPoint(WorldSpacePoint point, int index);
	
	public void setControlPoint1(WorldSpacePoint wsp) {
		controlPoint1 = wsp;
	}
	
	public void setControlPoint2(WorldSpacePoint wsp) {
		controlPoint2 = wsp;
	}

	public boolean isComplete() {
		return controlPoint2 != null;
	}
	
	public void setColor(Color c) {
		mainColor = c;
	}
	
	public abstract int numberChildrenNeeded();
	
	public abstract LineFonts getChildFont(int childIndex);
	
	public abstract Iterable<Line> getDrawLines();
	
	public abstract Iterable<DrawableString> getDrawableWords();
	
	public void deSelect() {
		setColor(UNSELECTION_COLOR);
		mouseClick = null;
		selectedPoint = -1;
	}
	
	public void select(Vector2 worldSpacePoint) {
		setColor(SELECTION_COLOR);
		mouseClick = worldSpacePoint;
		if(mouseClick.distanceTo(controlPoint1.getPoint()) < minDistanceForControlPointSelection) {
			selectedPoint = 1;
			//System.out.println("SketchoStatement: Selected control point 1");
		}
		else if(mouseClick.distanceTo(controlPoint2.getPoint()) < minDistanceForControlPointSelection) {
			selectedPoint = 2;
			//System.out.println("SketchoStatement: Selected control point 2");
		}
	}
	
	public WorldSpacePoint getControlPoint1 () {
		return controlPoint1;
	}
	
	public WorldSpacePoint getControlPoint2() {
		return controlPoint2;
	}
	
	public WorldSpacePoint getClosestControlPoint(Vector2 worldPoint) {
		if(worldPoint.distanceTo(controlPoint1.getPoint()) > worldPoint.distanceTo(controlPoint2.getPoint()))
			return controlPoint2;
		else
			return controlPoint1;
	}
	
	public boolean controlPoint1IsClosest(Vector2 otherPoint) {
		return otherPoint.distanceTo(controlPoint1.getPoint()) < otherPoint.distanceTo(controlPoint2.getPoint());
	}
	
	public abstract void doubleClick(Vector2 worldSpacePoint);
	
	public void mouseDrag(Vector2 offset, WorldSpacePoint newPoint) {
		if(selectedPoint == 1)
			setControlPoint1(newPoint);
		else if(selectedPoint == 2)
			setControlPoint2(newPoint);
	}
	
	/**
	 * Returns the shortest distance between p and a point in this figure.
	 * @param p - a point in screenspace
	 * @return The shortest distance between p and a point in this statement
	 */
	public float distanceToPoint(Vector2 p) {
		float minDistance = Float.MAX_VALUE;
		for(Line l : getDrawLines()) {
			float dist = l.distanceToPoint(p);
			if (dist < minDistance)
				minDistance = dist;
		}
		return minDistance;
	}
	
	public boolean equals(Object o) {
		return getId().equals(((SketchoStatement)o).getId());
	}
	
	public abstract boolean fitsSyntaxTree();
}
