package edu.wpi.aipaint.util;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * Vanilla implementation of a TreeNode with mutators.
 * 
 * Includes a generic user object.
 */
public class DefaultTreeNode<UserObjectType> implements UserObjectTreeNode<UserObjectType> {
	private UserObjectType userObject;
	
	private DefaultTreeNode<UserObjectType> parent;
	
	private List<DefaultTreeNode<UserObjectType>> children;
	
	public DefaultTreeNode()
	{
		userObject = null;
		parent = null;
		children = new LinkedList<DefaultTreeNode<UserObjectType>>();
	}
	
	public DefaultTreeNode(UserObjectType userObject){ 
		this.userObject = userObject;
		children = new LinkedList<DefaultTreeNode<UserObjectType>>();
	}
	
	public DefaultTreeNode(DefaultTreeNode<UserObjectType> parent, UserObjectType userObject) 
	{
		this.userObject = userObject;
		this.parent = parent;
		children = new LinkedList<DefaultTreeNode<UserObjectType>>();
	}
	
	public DefaultTreeNode(DefaultTreeNode<UserObjectType> parent, 
			List<DefaultTreeNode<UserObjectType>> children,
			UserObjectType userObject)
	{
		this.parent = parent;
		this.children = children;
		this.userObject = userObject;
	}
	
	@Override
	public int getChildCount() {
		return children.size();
	}

	@Override
	public Iterator<? extends TreeNode> getChildren() {
		return children.iterator();
	}
	
	public LinkedList<? extends TreeNode> getOrderedChildren() {
		return (LinkedList<? extends TreeNode>) children;
	}
	
	public void addChild(DefaultTreeNode<UserObjectType> newChild)
	{
		children.add(newChild);
	}

	@Override
	public DefaultTreeNode<UserObjectType> getParent() {
		return parent;
	}
	
	public void setParent(DefaultTreeNode<UserObjectType> parent)
	{
		this.parent = parent;
	}

	@Override
	public boolean isLeaf() {
		return children.size() == 0;
	}
	
	@Override
	public UserObjectType getUserObject()
	{
		return userObject;
	}
	
	@Override
	public <SubType extends UserObjectType> void setUserObject(SubType userObject)
	{
		this.userObject = userObject;
	}
	
	public void removeChild(DefaultTreeNode<UserObjectType> child) {
		children.remove(child);
	}
	
	@Override
	public boolean equals(Object o) {
		DefaultTreeNode<UserObjectType> other = (DefaultTreeNode<UserObjectType>) o;
		return(getUserObject().equals(other.getUserObject()));
	}
}
