package edu.wpi.aipaint.util;

import java.awt.Color;

public class Line {
	public enum LineFonts { 
		solid, dotted
	}
	
	private Vector2 point1, point2;
	private Color color;
	private LineFonts font;
	
	/*
	public Line(Vector2 point1, Vector2 point2) {
		this.point1 = point1;
		this.point2 = point2;
		color = Color.white;
	}
	*/
	
	public Line(Vector2 point1, Vector2 point2, Color c) {
		this.point1 = point1;
		this.point2 = point2;
		color = c;
		font = LineFonts.solid;
	}
	
	public Line(Vector2 point1, Vector2 point2, Color c, LineFonts font) {
		this.point1 = point1;
		this.point2 = point2;
		color = c;
		this.font = font;
	}
	
	public LineFonts getFont() {
		return font;
	}
	
	public Vector2 getPoint1() {
		return point1;
	}
	
	public Vector2 getPoint2() {
		return point2;
	}
	
	public Color getColor() {
		return color;
	}
	
	public void setColor(Color c) {
		color = c;
	}
	
	/**
	 * Returns the shortest distance from this line to the given point.
	 * @param p Point to measure distance again
	 * @return Distance, in screen space units
	 */
	public float distanceToPoint(Vector2 p) {
		
		//We define a line perpendicular to this line, using y = mx + b
		Vector2 perpendicularDirection = point1.rotate(point2, 90);
		perpendicularDirection = perpendicularDirection.subtract(point2);
		perpendicularDirection = perpendicularDirection.normalize();
		float intersectX;
		float m;
		float b;
		if(perpendicularDirection.getX() != 0 && point2.getX() - point1.getX() != 0) {
			float perp_m = perpendicularDirection.getY() / perpendicularDirection.getX();
			//use the known y and x, that is, points on the line to solve for b
			float perp_b = p.getY() - perp_m * p.getX();
			
			//now we need our own m and b
			m = (point2.getY() - point1.getY())/(point2.getX() - point1.getX());
			b = point2.getY() - m * point2.getX();
			
			//find the intersection point between the two lines
			intersectX = (b - perp_b) / (perp_m - m);
		} else if (perpendicularDirection.getX() == 0) {
			intersectX = p.getX();
			m = 0;
			b = point1.getY();
		} else {
			float intersectY = p.getY();
			intersectX = point1.getX();
			if(intersectY < Math.min(point1.getY(), point2.getY()) || intersectY > Math.max(point1.getY(), point2.getY())) {
				Vector2 candidate1 = point1.subtract(p);
				Vector2 candidate2 = point2.subtract(p);
				return Math.min(candidate1.getMagnitude(), candidate2.getMagnitude());
			} else {
				Vector2 shortest = p.subtract(new Vector2(intersectX, intersectY));
				return shortest.getMagnitude();
			}
		}
			
		//If the point is inside this line segment, distance is the distance from p to this point
		//Otherwise, distance is the shortest distance from this point to our end points
		if(intersectX < Math.min(point1.getX(), point2.getX()) || intersectX > Math.max(point1.getX(), point2.getX())) {
			//This point is not inside our line segment
			Vector2 candidate1 = point1.subtract(p);
			Vector2 candidate2 = point2.subtract(p);
			return Math.min(candidate1.getMagnitude(), candidate2.getMagnitude());
		} else {
			float intersectY = m * intersectX + b;
			Vector2 shortest = p.subtract(new Vector2(intersectX, intersectY));
			return shortest.getMagnitude();
		}		
	}
}
