package edu.wpi.aipaint.util;

import edu.wpi.aipaint.shapedetector.Point;

/**
 * Represents a two-dimensional vector.
 */
public class Vector2 {
	float x;
	float y;
	float magnitude;
	
	public static Vector2 UP = new Vector2(0, -1);
	public static Vector2 RIGHT = new Vector2(1, 0);
	public static Vector2 DOWN = new Vector2(0, 1);
	public static Vector2 LEFT = new Vector2(-1, 0);
	public static Vector2 ZERO = new Vector2(0, 0);
	
	public Vector2(float x, float y)
	{
		this.x = x;
		this.y = y;
		magnitude = (float) Math.sqrt(Math.pow(x, 2) + Math.pow(y, 2));
	}
	
	public Vector2(Point point) {
		this.x = point.X;
		this.y = point.Y;
	}

	public float getX()
	{
		return x;
	}
	
	public float getY()
	{
		return y;
	}
	
	// Add another vector to this one; return sum.
	public Vector2 add(Vector2 operand)
	{
		return new Vector2(x + operand.x, y + operand.y);
	}
	
	// Subtract another vector from this one; return sum.
	public Vector2 subtract(Vector2 operand)
	{
		return new Vector2(x - operand.x, y - operand.y);
	}
	
	// Dot another vector with this one; return product.
	public float dot(Vector2 operand)
	{
		return x * operand.x + y * operand.y;
	}
	
	// Rotate this vector around another vector that represents a pivot point; return result.
	// Angle of rotation is provided in degrees.
	public Vector2 rotate(Vector2 pivotPoint, float angle)
	{
		// Translate self to origin via pivotPoint.
		Vector2 translatedSelf = this.subtract(pivotPoint);
		
		// Rotate translatedSelf about origin.
		double angleRadians = Math.toRadians(angle);
		double cosine = Math.cos(angleRadians);
		double sine = Math.sin(angleRadians);
		Vector2 rotatedSelf = new Vector2((float) (translatedSelf.x * cosine - translatedSelf.y * sine), 
				(float) (translatedSelf.x * sine + translatedSelf.y * cosine));
		
		// Reverse translation.
		return rotatedSelf.add(pivotPoint);
	}
	
	// Love the given vector; return product.
	public Vector2 love(Vector2 operand) throws Exception
	{
		throw new Exception("There is no formula for love!");
	}
	
	// Return magnitude of this vector.
	public float getMagnitude()
	{
		return magnitude;
	}
	
	// Normalize this vector; return result.
	public Vector2 normalize()
	{
		float scaleFactor = 1 / Math.abs(magnitude);
		return new Vector2(x * scaleFactor, y * scaleFactor);
	}
	
	public float distanceTo(Vector2 other) {
		Vector2 distanceVector = subtract(other);
		return distanceVector.getMagnitude();
	}
	
	public Vector2 multiply(float scalar) 
	{
		return new Vector2(x * scalar, y * scalar);
	}
	
	public String toString() {
		return "(" + x + ", " + y + ")";
	}
	
	@Override
	public boolean equals(Object obj)
	{
		Vector2 other = (Vector2) obj;
		return other.getX() == x && other.getY() == y;
	}
	
	@Override
	public int hashCode()
	{
		return (x + " " + y).hashCode();
	}
}
