package edu.wpi.aipaint.worldspacepoint;

import edu.wpi.aipaint.attributes.Attribute;
import edu.wpi.aipaint.attributes.PoseableOffsetAttribute;
import edu.wpi.aipaint.main.Poseable;
import edu.wpi.aipaint.util.Vector2;

/**
 * Represents a point relative to a poseable. It follows the poseable's position and orientation, to for example always be in front of the poseable
 * or behind it.
 */
public class WorldSpacePointOffset implements WorldSpacePoint {
	Poseable base;
	float offsetAngle;
	float distance;
	
	public WorldSpacePointOffset(Poseable base, float offsetAngle, float distance) {
		this.base = base;
		this.offsetAngle = offsetAngle;
		this.distance = distance;
	}
	
	@Override
	public Vector2 getPoint() {
		Vector2 direction = base.getOrientation().rotate(Vector2.ZERO, offsetAngle);
		return base.getPosition().add(direction.multiply(distance));
	}

	@Override
	public float getX() {
		return getPoint().getX();
	}

	@Override
	public float getY() {
		return getPoint().getY();
	}

	@Override
	public boolean holdsPoseable() {
		return true;
	}

	@Override
	public Poseable getPoseable() {
		return base;
	}

	@Override
	public boolean holdsAttribute() {
		return false;
	}
	
	public Attribute translateToAttribute() {
		return new PoseableOffsetAttribute(base.getFeature(), base.getOrientationFeature(), offsetAngle, distance);
	}

	@Override
	public Attribute getAttribute() {
		return null;
	}

}
