package edu.wpi.aipaint.worldspacepoint;

import edu.wpi.aipaint.attributes.Attribute;
import edu.wpi.aipaint.main.Poseable;
import edu.wpi.aipaint.sketcho.SketchoStatement;
import edu.wpi.aipaint.util.Vector2;

/**
 * Represents another statement's control point location. Use it for following intermediate variables and such;
 */
public class WorldSpacePointSketchListener implements WorldSpacePoint {
	SketchoStatement parent;
	boolean useControlPoint2;
	
	public WorldSpacePointSketchListener(SketchoStatement parent, boolean useControlPoint2) {
		this.parent = parent;
		this.useControlPoint2 = useControlPoint2;
	}
	
	/**
	 * Makes a worldspacepointsketchlistener that follows the given sketch's controlpoint that is closest to basepoint at time of construction
	 * @param parent
	 * @param basePoint
	 */
	public WorldSpacePointSketchListener(SketchoStatement parent, Vector2 basePoint) {
		this.parent = parent;
		useControlPoint2 = !parent.controlPoint1IsClosest(basePoint);
	}
	
	@Override
	public Vector2 getPoint() {
		if(useControlPoint2) 
			return parent.getControlPoint2().getPoint();
		else
			return parent.getControlPoint1().getPoint();
	}
	
	private WorldSpacePoint getObservee() {
		if(useControlPoint2) {
			return parent.getControlPoint2();
		} else {
			return parent.getControlPoint1();
		}
	}

	@Override
	public float getX() {
		return getPoint().getX();
	}

	@Override
	public float getY() {
		return getPoint().getY();
	}

	@Override
	public boolean holdsPoseable() {
		return getObservee().holdsPoseable();
	}

	@Override
	public Poseable getPoseable() {
		return getObservee().getPoseable();
	}

	@Override
	public boolean holdsAttribute() {
		return getObservee().holdsAttribute();
	}

	@Override
	public Attribute getAttribute() {
		return getObservee().getAttribute();
	}
	
	public Attribute translateToAttribute() {
		return getObservee().translateToAttribute();
	}

}
