package aiPaintPacMan;

import java.awt.Dimension;
import java.awt.event.KeyListener;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.List;

import pacman.GameUI;
import pacman.PacMan;
import pacman.Thing;
import edu.wpi.aipaint.main.Game;
import edu.wpi.aipaint.main.NullMouseListener;
import edu.wpi.aipaint.main.NullMouseMotionListener;
import edu.wpi.aipaint.main.Poseable;
import edu.wpi.aipaint.main.WorldState;
import edu.wpi.aipaint.pipeline.AIAgent;
import edu.wpi.aipaint.util.Vector2;

public class AIPaintPacMan extends PacMan implements Game {

	private static final long serialVersionUID = 5571164952078732475L;
	WorldState m_lastState;
	
	Dimension m_dim;
	float percentToInterfaceDivider;
	float percentToTopAndBottomDivider;

	public AIPaintPacMan() {
		super();
		m_lastState = new WorldState();
		m_dim = new Dimension(609, 450); //Pacman's default dimension
	}
	
	@Override
	public void aiPaintInit() {
	   // Create canvases and layout
	   init();
	   for(Thing tt : m_gameModel.getM_things()) {
		   ((AiPacmanInitializable)tt).setWidthAndHeight(this);
	   }
	   
	   
   }
	
	public GameUI getGameUI() {
		return this.m_gameUI;
	}
	
	public Thing[] getM_things() {
		return this.m_gameModel.getM_things();
	}

	@Override
	public void draw(int gameTime) {
		draw();

	}

	@Override
	public void tick(int gameTime) {
		tick();
		
		WorldState ws = this.m_lastState.copy();

		for (Thing tt : m_gameModel.getM_things()) {
			if (tt instanceof Poseable) {
				ws.addFeature(((Poseable) tt).getFeature(), ((Poseable) tt).getFeatureValue());
				ws.addFeature(((Poseable) tt).getOrientationFeature(), ((Poseable) tt).getOrientationFeatureValue());
			}
		}
		
		this.m_lastState = ws;
	}

	@Override
	public KeyListener getKeyboardListener() {
		if (this.getKeyListeners().length > 0)
			return this.getKeyListeners()[0];
		else
			return null;
	}
	
	public MouseListener getMouseListener() {
		return new NullMouseListener();
	}
	
	public MouseMotionListener getMouseMotionListener() {
		return new NullMouseMotionListener();
	}

	@Override
	public Poseable getPoseableAt(Vector2 worldSpacePoint) {
	    for (Thing tt : m_gameModel.getM_things()) {
	        if (Math.abs(tt.getM_locX() - worldSpacePoint.getX()) < .9 && Math.abs(tt.getM_locY() - worldSpacePoint.getY()) < .9) {
	            return (Poseable)tt;
	        }
	    }
	    return null;
	}
	
	@Override
	public AIAgent getAIAgentAt(Vector2 worldSpacePoint)
	{
		for(Thing t : this.m_gameModel.getM_things()){
			if(t instanceof AIAgent){
				if (Math.abs(t.getM_locX() - worldSpacePoint.getX()) < .9 && Math.abs(t.getM_locY() - worldSpacePoint.getY()) < .9) {
					return (AIAgent) t;
				}
			}
		}
		return null;
	}
	
	public List<Poseable> getPoseablesCollidingWith(Vector2 worldSpacePoint, float worldSpaceRadius) {
		float minX = worldSpacePoint.getX() - worldSpaceRadius;
		float maxX = worldSpacePoint.getX() + worldSpaceRadius;
		float minY = worldSpacePoint.getY() - worldSpaceRadius;
		float maxY = worldSpacePoint.getY() + worldSpaceRadius;
		ArrayList<Poseable> poseables = new ArrayList<Poseable>();
		for(Thing tt : m_gameModel.getM_things()) {
			if(between(tt.getM_locX(), minX, maxX) && between(tt.getM_locY(), minY, maxY) && tt.available())
				poseables.add((Poseable) tt);
		}
		return poseables;
	}
	
	private boolean between(float p, float a, float b) {
		return p <= b && p >= a;
	}

	@Override
	public Vector2 getWorldSpacePoint(Vector2 screenSpacePoint) {
		float scaleX = (float)(getSize().getWidth()/m_dim.getWidth());
		float scaleY = (float)(getSize().getHeight()/m_dim.getHeight());
		
		int cell_length = m_gameUI.getCELL_LENGTH();
		int offset = m_gameUI.getM_gridInset();
		Vector2 point = new Vector2((screenSpacePoint.getX() - offset * scaleX) / (cell_length * scaleX),
				(screenSpacePoint.getY() - offset * scaleY) / (cell_length * scaleY));
		return point;
	}

	@Override
	public WorldState getCurrentWorldState() {
		return this.m_lastState;
	}

	@Override
	public Vector2 getScreenSpacePoint(Vector2 worldSpacePoint) {
		float scaleX = (float)(getSize().getWidth()/m_dim.getWidth());
		float scaleY = (float)(getSize().getHeight()/m_dim.getHeight());
		
		int cell_length = m_gameUI.getCELL_LENGTH();
		int offset = m_gameUI.getM_gridInset();
		
		Vector2 point = new Vector2(worldSpacePoint.getX() * cell_length * scaleX + offset * scaleX, worldSpacePoint.getY() * cell_length * scaleY + offset * scaleY);
		//System.out.println("World: " + worldSpacePoint + ", point: " + point + "Scale: " + new Vector2(scaleX, scaleY));
		
		return point;
	}
	
	@Override
	public void setSize(Dimension d) {
		super.setSize(d);
		
		float scaleX = (float)(getSize().getWidth()/m_dim.getWidth());
		float scaleY = (float)(getSize().getHeight()/m_dim.getHeight());
		m_gameUI.setSize(new Dimension((int)(m_gameUI.getResolutionIndependentDimensions().getWidth() * scaleX), (int)(m_gameUI.getResolutionIndependentDimensions().getHeight() * scaleY)));
		m_gameUI.setBounds(0, 0, (int)m_gameUI.getSize().getWidth(), (int)m_gameUI.getSize().getHeight());
		m_topCanvas.setSize(new Dimension((int)(m_topCanvas.getResolutionIndependentDimensions().getWidth() * scaleX), (int)(m_topCanvas.getResolutionIndependentDimensions().getHeight() * scaleY)));
		m_topCanvas.setBounds((int)m_gameUI.getSize().getWidth(), 0, (int)m_topCanvas.getSize().getWidth(), (int)m_topCanvas.getSize().getHeight());
		m_bottomCanvas.setSize(new Dimension((int)(m_bottomCanvas.getResolutionIndependentDimensions().getWidth() * scaleX), (int)(m_bottomCanvas.getResolutionIndependentDimensions().getHeight() * scaleY)));
		m_bottomCanvas.setBounds((int)m_gameUI.getSize().getWidth(), (int)m_topCanvas.getSize().getHeight(), (int)m_bottomCanvas.getSize().getWidth(), (int)m_bottomCanvas.getSize().getHeight());
		
	}
}
