package aiPaintPacMan;

import java.util.Hashtable;
import java.util.LinkedList;


import edu.wpi.aipaint.attributes.Attribute;
import edu.wpi.aipaint.attributes.ClosestAttribute;
import edu.wpi.aipaint.attributes.PassThroughAttribute;
import edu.wpi.aipaint.main.Group;
import edu.wpi.aipaint.main.GroupManager;
import edu.wpi.aipaint.main.WorldFeature;
import edu.wpi.aipaint.sketcho.HeuristicFactory;
import edu.wpi.aipaint.worldspacepoint.WorldSpacePoint;
import edu.wpi.aipaint.worldspacepoint.WorldSpacePointAttribute;

public class PacManHeuristicFactory implements HeuristicFactory {
	private Hashtable<Group, LinkedList<Attribute>> groupAttributeMap;
	
	public PacManHeuristicFactory()
	{
		groupAttributeMap = new Hashtable<Group, LinkedList<Attribute>>();
		
		LinkedList<Group> spatialGroups = new LinkedList<Group>();
		LinkedList<Group> otherGroups = new LinkedList<Group>();
		GroupManager groupManager = GroupManager.GetInstance();
		spatialGroups.add(groupManager.createGroup("powerpellet"));
		otherGroups.add(groupManager.createGroup("pickup"));
		otherGroups.add(groupManager.createGroup("ghost"));
		otherGroups.add(groupManager.createGroup("fruit"));
		
		Attribute pacManAttribute = new PassThroughAttribute(
				new WorldFeature("PoseablePlayerPosition"));
		WorldSpacePoint pacManPosition = new WorldSpacePointAttribute(pacManAttribute);
		
		Attribute ghost0Attribute = new PassThroughAttribute(
				new WorldFeature("PoseableGhost_0_Position"));
		WorldSpacePoint ghost0Position = new WorldSpacePointAttribute(ghost0Attribute);
		
		for (Group spatialGroup : spatialGroups)
		{
			LinkedList<Attribute> spatialAttributes = new LinkedList<Attribute>();
			
			spatialAttributes.add(new ClosestAttribute(pacManPosition, spatialGroup));
			spatialAttributes.add(new ClosestAttribute(ghost0Position, spatialGroup));
			
			groupAttributeMap.put(spatialGroup, spatialAttributes);
		}
		
		for (Group otherGroup : otherGroups)
		{
			groupAttributeMap.put(otherGroup, new LinkedList<Attribute>());
		}
	}

	@Override
	public Iterable<Attribute> getGeneralizingAttributes(Group group) {
		return groupAttributeMap.get(group);
	}
}
