package aiPaintPacMan;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import actions.GoToAttributeAction;
import edu.wpi.aipaint.ai.BTLeafNode;
import edu.wpi.aipaint.attributes.Attribute;
import edu.wpi.aipaint.directo.AttachChildDirection;
import edu.wpi.aipaint.pipeline.Direction;
import edu.wpi.aipaint.sketcho.SketchoActionStatement;
import edu.wpi.aipaint.util.Line;
import edu.wpi.aipaint.util.Vector2;
import edu.wpi.aipaint.worldspacepoint.WorldSpacePoint;
import edu.wpi.aipaint.worldspacepoint.WorldSpacePointPoseable;

/**
 * SketchoStatement that expresses a PacMan GoToWorldSpacePointAction. :)
 */
public class PacManMoveToSketchoActionStatement extends SketchoActionStatement {
    
	public PacManMoveToSketchoActionStatement()
	{
	    this.setControlPoint1(new WorldSpacePointPoseable(PacManForPaintMain.getPipelineModule().getAIAgent()));
	}
	
	@Override
	public void doubleClick(Vector2 worldSpacePoint) { }

	@Override
	public Iterable<Line> getDrawLines() {
		Vector2 cp1 = controlPoint1.getPoint();
		Vector2 cp2 = controlPoint2.getPoint();
		
		if(Math.abs(cp1.getX() - cp2.getX()) < .01 && Math.abs(cp1.getY() - cp2.getY()) < .01)
			return new ArrayList<Line>();
		
		Line[] drawLine = new Line[3];		
		drawLine[0] = new Line(cp1, cp2, mainColor);
		
		float dist = translator.screenSpaceToWorldSpace(new Vector2(20, 0)).subtract(translator.screenSpaceToWorldSpace(new Vector2(0, 0))).getMagnitude();
		
		Vector2 arrowHead1 = cp1.rotate(cp2, 45);		
		arrowHead1 = arrowHead1.subtract(cp2);
		arrowHead1 = arrowHead1.normalize();
		arrowHead1 = arrowHead1.multiply(dist);
		arrowHead1 = arrowHead1.add(cp2);
		drawLine[1] = new Line(cp2, arrowHead1, mainColor);
		
		Vector2 arrowHead2 = cp1.rotate(cp2, -45);
		arrowHead2 = arrowHead2.subtract(cp2);
		arrowHead2 = arrowHead2.normalize();
		
		arrowHead2 = arrowHead2.multiply(dist);
		arrowHead2 = arrowHead2.add(cp2);
		drawLine[2] = new Line(cp2, arrowHead2, mainColor);
		
		return Arrays.asList(drawLine);
	}

	@Override
	public List<Direction> translate() {
		List<Direction> translation = new LinkedList<Direction>();
		Attribute target = controlPoint2.translateToAttribute();

		BTLeafNode moveToActionNode = new BTLeafNode(null, new GoToAttributeAction(target));
		AttachChildDirection addActionNode = new AttachChildDirection(moveToActionNode);
		
		translation.add(addActionNode);
		
		return translation;
	}

	@Override
	public Iterable<WorldSpacePoint> getGeneralizableControlPoints() {
		LinkedList<WorldSpacePoint> controlPoints = new LinkedList<WorldSpacePoint>();
		controlPoints.add(controlPoint2);
		return controlPoints;
	}

	@Override
	public void setGeneralizableControlPoint(WorldSpacePoint point, int index) {
		controlPoint2 = point;
	}

}
