package aiPaintPacMan;

import actions.GoToAttributeAction;
import actions.GoTwiceAsFarAttributeAction;
import edu.wpi.aipaint.attributes.Attribute;
import edu.wpi.aipaint.attributes.DistanceAttribute;
import edu.wpi.aipaint.pipeline.Action;
import edu.wpi.aipaint.sketcho.AreaConditionalStatement;
import edu.wpi.aipaint.sketcho.DistanceConditionalStatement;
import edu.wpi.aipaint.sketcho.SketchoIntermediateVariable;
import edu.wpi.aipaint.sketcho.SketchoStatement;
import edu.wpi.aipaint.sketcho.StatementFactory;
import edu.wpi.aipaint.worldspacepoint.WorldSpacePoint;
import edu.wpi.aipaint.worldspacepoint.WorldSpacePointAttribute;

/**
 * PacMan implementation of the AIPaint StatementFactory interface.
 * Produces SketchoStatements with PacMan Actions.
 */
public class PacManStatementFactory implements StatementFactory {
	@Override
	public SketchoStatement getStatement(String statementName) {
		if (statementName.equals("Area"))
			return new AreaConditionalStatement();
		else if (statementName.equals("DistanceTo"))
			return new DistanceConditionalStatement();
		else if (statementName.equals("MoveTo"))
			return new PacManMoveToSketchoActionStatement();
		else if(statementName.equals("Variable"))
			return new SketchoIntermediateVariable();
		else if(statementName.equals("GoTwiceAsFar"))
			return new PacManGoTwiceAsFarSketchoActionStatement();
		return null;
	}

	@Override
	public SketchoStatement getStatement(Action actionProduced) {
		if (actionProduced instanceof GoToAttributeAction)
		{
			GoToAttributeAction action = (GoToAttributeAction) actionProduced;
			SketchoStatement statement = new PacManMoveToSketchoActionStatement();
			WorldSpacePoint target = new WorldSpacePointAttribute(action.getTarget());
			statement.setGeneralizableControlPoint(target, 0);
			return statement;
		} else if (actionProduced instanceof GoTwiceAsFarAttributeAction) {
			GoTwiceAsFarAttributeAction action = (GoTwiceAsFarAttributeAction)actionProduced;
			SketchoStatement statement = new PacManGoTwiceAsFarSketchoActionStatement();
			WorldSpacePoint source = new WorldSpacePointAttribute(action.getStart());
			statement.setControlPoint1(source);
			
			WorldSpacePoint end = new WorldSpacePointAttribute(action.getEnd());
			statement.setControlPoint2(end);
			return statement;
		}
		return null;
	}

	@Override
	public SketchoStatement getStatement(Attribute attributeProduced) {
		if (attributeProduced instanceof DistanceAttribute)
		{
			DistanceAttribute attribute = (DistanceAttribute) attributeProduced;
			DistanceConditionalStatement statement = new DistanceConditionalStatement();
			WorldSpacePoint point1 = new WorldSpacePointAttribute(attribute.getPosition1());
			WorldSpacePoint point2 = new WorldSpacePointAttribute(attribute.getPosition2());
			statement.setGeneralizableControlPoint(point1, 0);
			statement.setGeneralizableControlPoint(point2, 1);
			statement.setThreshold(attribute.getThreshold());
			return statement;
		}
		return null;
	}
}
