package aiPaintPacMan;

import java.awt.Color;
import java.awt.Point;
import java.util.LinkedList;

import pacman.GameModel;
import pacman.Ghost;
import actions.MoveAction;
import edu.wpi.aipaint.main.Game;
import edu.wpi.aipaint.main.Group;
import edu.wpi.aipaint.main.GroupManager;
import edu.wpi.aipaint.main.Poseable;
import edu.wpi.aipaint.main.WorldFeature;
import edu.wpi.aipaint.main.WorldState;
import edu.wpi.aipaint.pipeline.AIAgent;
import edu.wpi.aipaint.pipeline.AIController;
import edu.wpi.aipaint.pipeline.Action;
import edu.wpi.aipaint.util.Vector2;
import edu.wpi.aipaint.values.Value;
import edu.wpi.aipaint.values.Vector2Wrapper;

public class PoseableGhost extends Ghost implements Poseable, AIAgent, AiPacmanInitializable {
	GhostAIController m_ai;
	int m_id;
	private WorldFeature position, orientation;
	
	private LinkedList<Group> groups;

	public PoseableGhost(GameModel gameModel, byte type, int startX,
			int startY, boolean bMiddle, Color color, int nExitMilliSec, int iD) {
		super(gameModel, type, startX, startY, bMiddle, color, nExitMilliSec);

		// default behavior is to flee the player
		this.m_ai = new GhostAIController(this);

		this.m_id = iD;

		position = new WorldFeature("PoseableGhost_" + m_id + "_Position");
		orientation = new WorldFeature("PoseableGhost_" + m_id + "_Orientation");
		
		groups = new LinkedList<Group>();
		GroupManager groupManager = GroupManager.GetInstance();
		groups.add(groupManager.registerPoseable(this, "ghost"));
	}

	/**
	 * Sets the target for setNextDirection. Calls AI module
	 */
	private Point chooseTarget() {
		Point p = new Point();
		if (this.m_bInsideRoom) {
			p.x = this.m_gameModel.m_doorLocX;
			p.y = m_gameModel.m_doorLocY;
		} else if (m_bEaten) {
			// If the ghost is eaten, it needs to return to the hideout.
			p.x = m_gameModel.m_doorLocX;
			p.y = m_gameModel.m_doorLocY - 1;
		} else {
			WorldState presentWorldState = 
				((AIPaintPacMan) this.m_gameModel.m_pacMan).getCurrentWorldState();
			Action todo = this.m_ai.evaluate(presentWorldState);
			if (todo instanceof MoveAction) {
				MoveAction moveAction = (MoveAction) todo;
				return moveAction.getDestination(this, presentWorldState);
			}
		}
		return p;
	}

	/**
	 * Finds the best directions to go in
	 * 
	 * @param target
	 *            Point to go to
	 * @param bBackoff
	 *            If the cycle is fleeing
	 * @return Array of directions to move in, in order of preference
	 */
	private byte[] getBestDirections(Point target, boolean bBackoff) {
		int deltaX, deltaY;
		byte[] bestDirection = new byte[4];

		deltaX = getM_locX() - target.x;
		deltaY = getM_locY() - target.y;

		if (Math.abs(deltaX) > Math.abs(deltaY)) {
			if (deltaX > 0) {
				bestDirection[0] = LEFT;
				bestDirection[3] = RIGHT;
				if (deltaY > 0) {
					bestDirection[1] = UP;
					bestDirection[2] = DOWN;
				} else {
					bestDirection[1] = DOWN;
					bestDirection[2] = UP;
				}
			} else {
				bestDirection[0] = RIGHT;
				bestDirection[3] = LEFT;
				if (deltaY > 0) {
					bestDirection[1] = UP;
					bestDirection[2] = DOWN;
				} else {
					bestDirection[1] = DOWN;
					bestDirection[2] = UP;
				}
			}
		} else {
			if (deltaY > 0) {
				bestDirection[0] = UP;
				bestDirection[3] = DOWN;
				if (deltaX > 0) {
					bestDirection[1] = LEFT;
					bestDirection[2] = RIGHT;
				} else {
					bestDirection[1] = RIGHT;
					bestDirection[2] = LEFT;
				}

			} else {
				bestDirection[0] = DOWN;
				bestDirection[3] = UP;
				if (deltaX > 0) {
					bestDirection[1] = LEFT;
					bestDirection[2] = RIGHT;
				} else {
					bestDirection[1] = RIGHT;
					bestDirection[2] = LEFT;
				}
			}
		}

		// There's a 50% chance that the ghost will try the sub-optimal
		// direction first.
		// This will keep the ghosts from following each other and to trap
		// Pacman.
		if (!this.m_bInsaneAI && this.m_bCanUseNextBest && Math.random() < .50) {
			byte temp = bestDirection[0];
			bestDirection[0] = bestDirection[1];
			bestDirection[1] = temp;
		}

		// If the ghost is fleeing and not eaten, then reverse the array of best
		// directions to go.
		if (bBackoff || (this.m_nTicks2Flee > 0 && !this.m_bEaten)) {
			byte temp = bestDirection[0];
			bestDirection[0] = bestDirection[3];
			bestDirection[3] = temp;

			temp = bestDirection[1];
			bestDirection[1] = bestDirection[2];
			bestDirection[2] = temp;
		}

		return bestDirection;
	}

	@Override
	public WorldFeature getFeature() {
		return position;
	}

	@SuppressWarnings("unchecked")
	@Override
	public <ValueType extends Value> ValueType getFeatureValue() {
		return (ValueType) new Vector2Wrapper(getPosition());
	}

	@Override
	public AIController getAIController() {
		return this.m_ai;
	}

	@Override
	public Iterable<Group> getGroups() {
		return groups;
	}

	@Override
	public Vector2 getPosition() {
		return new Vector2(this.getM_locX() + width/2.0f, this.getM_locY() + height/2.0f);
	}

	@Override
	public void moveTo(Vector2 worldSpacePoint) {
		setM_locX((int) worldSpacePoint.getX());
		setM_locY((int) worldSpacePoint.getY());
	}

	/**
	 * Sets the direction to go in the agent itself
	 * 
	 * @param prevDirection
	 *            Direction that the AI is coming from to prevent doubling back
	 * @param bestDirection
	 *            Sorted list of directions to go in
	 */
	private void setDirection(byte prevDirection, byte[] bestDirection) {
		Point nextLocation = new Point();
		for (int i = 0; i < 4; i++) {
			if (bestDirection[i] == UP
					&& (m_gameModel.m_gameState[getM_locX()][getM_locY()] & GameModel.GS_NORTH) == 0
					&& m_deltaLocX == 0 && prevDirection != DOWN) {
				if (!getDestination(UP, getM_locX(), getM_locY(), nextLocation))
					continue;
				m_destinationX = nextLocation.x;
				m_destinationY = nextLocation.y;
				m_direction = UP;
				if (m_bCanFollow || !isFollowing())
					break;

			} else if (bestDirection[i] == DOWN
					&& (m_gameModel.m_gameState[getM_locX()][getM_locY()] & GameModel.GS_SOUTH) == 0
					&& m_deltaLocX == 0 && prevDirection != UP) {
				if (!getDestination(DOWN, getM_locX(), getM_locY(),
						nextLocation))
					continue;
				m_destinationX = nextLocation.x;
				m_destinationY = nextLocation.y;
				m_direction = DOWN;
				if (m_bCanFollow || !isFollowing())
					break;

			} else if (bestDirection[i] == RIGHT
					&& (m_gameModel.m_gameState[getM_locX()][getM_locY()] & GameModel.GS_EAST) == 0
					&& m_deltaLocY == 0 && prevDirection != LEFT)

			{
				if (!getDestination(RIGHT, getM_locX(), getM_locY(),
						nextLocation))
					continue;
				m_destinationX = nextLocation.x;
				m_destinationY = nextLocation.y;
				m_direction = RIGHT;
				if (m_bCanFollow || !isFollowing())
					break;

			} else if (bestDirection[i] == LEFT
					&& (m_gameModel.m_gameState[getM_locX()][getM_locY()] & GameModel.GS_WEST) == 0
					&& m_deltaLocY == 0 && prevDirection != RIGHT) {
				if (!getDestination(LEFT, getM_locX(), getM_locY(),
						nextLocation))
					continue;
				m_destinationX = nextLocation.x;
				m_destinationY = nextLocation.y;
				m_direction = LEFT;
				if (m_bCanFollow || !isFollowing())
					break;
			}
		}
	}

	@Override
	public void setNextDirection(byte prevDirection, boolean bBackoff) {
		Point target = chooseTarget();
		byte[] bestDirection = getBestDirections(target, bBackoff);
		setDirection(prevDirection, bestDirection);
		if (this.m_id == 0) {
			System.out.println("-------------AI REP FOR " + this.m_color.toString() + " ---------------");
			System.out.println(this.m_ai.getBehaviorTree().printTree());
			System.out.println("    Active Node: ");
			System.out.println(this.m_ai.getBehaviorTree().getActiveNode(
					((AIPaintPacMan) this.m_gameModel.m_pacMan)
							.getCurrentWorldState()));
			System.out.println("-----------------------------------------");
		}
	}

	public String toString() {
		return "PoseableGhost [id=\"" + this.m_id + "\"; color=\""
				+ this.m_color + "\"]";
	}
	
	@Override
	public void setWidthAndHeight(Game init) {
		//Vector2 widthVec = init.getWorldSpacePoint(new Vector2(0, 0).subtract(new Vector2(((AIPaintPacMan)init).getGameUI().getCELL_LENGTH(), 0)));
		width = ((AIPaintPacMan)init).getGameUI().getCELL_LENGTH() / 8;
		height = width;	
		
	}

	@Override
	public Vector2 getOrientation() {
		if(m_direction == UP) {
			return Vector2.UP;
		} else if (m_direction == RIGHT) {
			return Vector2.RIGHT;
		} else if (m_direction == DOWN) {
			return Vector2.DOWN; 
		} else {
			return Vector2.LEFT;
		}
	}
	
	@Override
	public WorldFeature getOrientationFeature() {
		return orientation;
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public <ValueType extends Value> ValueType getOrientationFeatureValue() {
		return (ValueType) new Vector2Wrapper(getOrientation());
	}
}
