package pacman;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Image;

import javax.swing.JPanel;

//Bottom right canvas that is repainted infrequently.
//It contains the number of lives, level and instructions
public class BottomCanvas extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	Font m_font;
	GameModel m_gameModel;
	PacMan m_pacMan;

	Image m_offImage;
	Graphics m_offGraphics;
	Dimension m_offDim;
	Dimension m_dim;

	public BottomCanvas(PacMan pacMan, GameModel gameModel, int width,
			int height) {
		super();
		// setSize (width, height);
		setPreferredSize(new Dimension(width, height));
		m_gameModel = gameModel;
		m_pacMan = pacMan;
		m_font = new Font("Helvetica", Font.BOLD, 18);
		m_dim = new Dimension(width, height);
	}

	public void update(Graphics g) {
		paint(g);
	}

	public Dimension getResolutionIndependentDimensions() {
		return m_dim;
	}

	public void paintComponent(Graphics g) {
		super.paintComponent(g);

		Image draw = getOffImage(g);

		// Blitz buffer into actual graphic
		g.drawImage(draw, 0, 0, this);
	}

	public Image getOffImage(Graphics g) {
		Dimension dim = getSize();

		// Create double buffer if it does not exist or is not
		// the right size
		if (m_offImage == null || m_offDim.width != dim.width
				|| m_offDim.height != dim.height) {
			m_offDim = dim;
			m_offImage = createImage(m_offDim.width, m_offDim.height);
			m_offGraphics = m_offImage.getGraphics();
		}

		double pacManDiameter = m_offDim.height / 15;
		int x = 0;
		int y = 0;
		String stageString = "Level " + Integer.toString(m_gameModel.m_stage);

		m_offGraphics.setColor(Color.black);
		m_offGraphics.fillRect(0, 0, m_offDim.width, m_offDim.height);

		m_offGraphics.setFont(m_font);
		FontMetrics fm = m_offGraphics.getFontMetrics();

		y = fm.getAscent() + fm.getDescent();
		x = 0;
		m_offGraphics.setColor(Color.white);
		m_offGraphics.drawString(stageString, x, y);

		y += fm.getAscent() + fm.getDescent();
		m_offGraphics.setColor(Color.yellow);
		for (int i = 0; i < m_gameModel.m_nLives; i++) {
			m_offGraphics.fillArc(x, y, (int) pacManDiameter,
					(int) pacManDiameter, 45, 200);
			m_offGraphics.fillArc(x, y, (int) pacManDiameter,
					(int) pacManDiameter, -45, -200);
			x += pacManDiameter * 1.5;
		}

		m_offGraphics.setColor(Color.white);
		y += 2 * (int) pacManDiameter + fm.getAscent() + fm.getDescent();
		x = 0;
		m_offGraphics.drawString("Use Arrow Keys", x, y);

		y += fm.getAscent() + fm.getDescent();
		m_offGraphics.drawString("\'N\' for New Game", x, y);

		y += fm.getAscent() + fm.getDescent();
		m_offGraphics.drawString("\'P\' to Pause", x, y);
		
		y += fm.getAscent() + fm.getDescent();
		m_offGraphics.drawString("\'K\' for AIPaint", x, y);

		y += fm.getAscent() + fm.getDescent();
		m_offGraphics.drawString("\'A\' for About", x, y);

		y += fm.getAscent() + fm.getDescent();
		if (m_pacMan.m_soundMgr != null && m_pacMan.m_soundMgr.soundEnabled())
			m_offGraphics.drawString("\'S\' for No Sound", x, y);
		else
			m_offGraphics.drawString("\'S\' for Sound", x, y);

		// y += fm.getAscent() + fm.getDescent();
		// m_offGraphics.drawString ("\'I\'  for Insane AI", x, y);
		return m_offImage;
	}
}
